runanalysis = function(){

#copy static files
flistcss <- list.files(paste(path.pipeline.base,"//css",sep=""),"^.+[.]css$",full.names=TRUE)
invisible(file.copy(flistcss,path.css))
flistjs <- list.files(paste(path.pipeline.base,"//js",sep=""),"^.+[.]js$",full.names=TRUE)
invisible(file.copy(flistjs,path.js))
flistimg <- list.files(paste(path.pipeline.base,"//img",sep=""),"^.+[.].+$",full.names=TRUE)
invisible(file.copy(flistimg,path.img))


summarize.arguments.file(outfile = paste(getwd(),"//arguments_summary.txt",sep=""))
summarize.arguments.file(outfile = paste(path.inc,"//arguments.js",sep=""))

##### apply make.name() to arguments that are column headers:
if(length(sampleannot.variables)>0){sampleannot.variables=make.names(sampleannot.variables)}
if(length(deregmodule.arg.is.normal.variable)>0){deregmodule.arg.is.normal.variable=make.names(deregmodule.arg.is.normal.variable)}
if(length(deregmodule.arg.covariates)>0){deregmodule.arg.covariates=make.names(deregmodule.arg.covariates)}
if(length(deregmodule.arg.adjust.for)>0){deregmodule.arg.adjust.for=make.names(deregmodule.arg.adjust.for)}
if(length(DEmodule.arg.predictors)>0){DEmodule.arg.predictors=make.names(DEmodule.arg.predictors)}
if(length(DEmodule.arg.confounders)>0){DEmodule.arg.confounders=make.names(DEmodule.arg.confounders)}


##### load data and annotation:
cat("loading annotation files",file=log,sep='\n\n',append=TRUE)
# load w/o row.names
annot = read.csv(path.sampleannot,row.names=sample.annotation.id.column) #changed var to sample.annotation.id.column as that is hardcoded in nSolver

cat("loading gene annotation files",file=log,sep='\n\n',append=TRUE)

gannot = read.csv(path.geneannot)
suppressWarnings(write.table(gannot[1:5,1:5],file=log,sep='\t',append=TRUE))
gannot.remove = !is.element(tolower(gannot$Class.Name),c("endogenous","housekeeping"))
cat(paste(table(gannot.remove),sep='\t'),file=log,sep='\n',append=TRUE)
cat(paste(summary(gannot.remove),sep='\t'),file=log,sep='\n',append=TRUE)
gannot = gannot[!gannot.remove,]
dimnames(gannot)[[1]] = gannot[,gene.annotation.id.column]
# for cancer pathways panel analyses, all NA gene annotation values can be set to 0:
replace.gannot.NA.with.0 = TRUE
if(replace.gannot.NA.with.0){ gannot = replace(gannot,is.na(gannot),0) }
cat("loaded gene annotation",file=log,sep='\n\n',append=TRUE)

cat("loading data",file=log,sep='\n\n',append=TRUE)
if(!exists("data.type")){data.type = "normalized"}   
if(data.type=="normalized"){raw = read.csv(path.data[1],row.names=1)}
if(data.type=="raw"){raw = read.csv(path.data[2],row.names=1)}

pruneflag=FALSE

## screen out low-count genes:
if((data.type=="raw")&prunegenes.run)
{
  pruneflag=TRUE
  prune = apply((raw<prunegenes.bg.level),2,mean,na.rm=T)>prunegenes.missing.freq
  prune[is.na(prune)]=TRUE
  if(mean(prune)>0.8)
  {
    pruneflag=FALSE
    warning("Warning: More than 80% of genes were pruned using these cutoffs - retaining all genes")
    cat("LOG:Warning: Sample IDs in sample annotation file don't mathc sample IDs in expression dataset",file=log,sep='\n',append=TRUE)
    cat("document.write('<p>Warning: More than 80% of genes were pruned using these cutoffs - retaining all genes</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
    prune = FALSE
  }
  prunedgenes = suppressWarnings(as.matrix(dimnames(raw)[[2]][prune])); 
  colnames(prunedgenes)=paste("The below genes were removed for falling below the background level of",
                              prunegenes.bg.level,"with frequency greater than",prunegenes.missing.freq)
  write.csv(as.matrix(prunedgenes),file="genes removed for low signal.csv",row.names=FALSE)
  raw = raw[,!prune]
}


cat("dimensions of gannot:",file=log,sep='\n',append=TRUE)
cat(dim(gannot),file=log,sep='\n',append=TRUE)
cat("dimensions of annot:",file=log,sep='\n',append=TRUE)
cat(dim(annot),file=log,sep='\n',append=TRUE)
cat("dimensions of raw:",file=log,sep='\n',append=TRUE)
cat(dim(raw),file=log,sep='\n',append=TRUE)

## exit if raw has only one sample:
if(dim(raw)[1]==1)
{
  cat("document.write('<p>Only one sample was included in the analysis - Re-run with multiple samples</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
  stop("Only one sample was included in the analysis.  Re-run with multiple samples.")
}

print("Loaded user files")  
cat("LOG:Loaded user files",file=log,sep='\n\n',append=TRUE)
cat("document.write('<p>Loaded user files</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
names(sampleannot.variabletypes)=sampleannot.variables
plottypearg = unique(c("png",plottypearg))

runloadlibraries = function(){
  print("Starting loading R libraries")  
  cat("LOG:Starting loading R libraries",file=log,sep='\n\n',append=TRUE)
  cat("document.write('<p>Starting loading R libraries</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
  ### install packages and load libraries
  path.libs = paste(path.pipeline.base,"//libs",sep="")
  cat("installing libraries",file=log,sep='\n',append=TRUE)
  #getpackages(path.libs)
  downloadpackages(path.libs)
  cat("loading libraries",file=log,sep='\n',append=TRUE)
  requirelibraries()
}
resultlib<-suppressWarnings(try(runloadlibraries(), silent = TRUE))
warnings()
if(class(resultlib) == "try-error"){
  cat("LOG:could not load libraries",file=log,sep='\n',append=TRUE)
  cat(resultlib,file=log,sep='\n',append=TRUE)
  cat("document.write('<p>Could not load libraries</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
  stop("could not load libraries")
}else{
  print("Finished loading R libraries")
  cat("LOG:Finished loading R libraries",file=log,sep='\n\n',append=TRUE)
  cat("document.write('<p>Finished loading R libraries</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE) 
}
print("Finished setup; starting annotations and data checking")
cat("LOG:Finished setup; starting annotations and data checking",file=log,sep='\n\n',append=TRUE)
cat("document.write('<p>Finished setup; starting annotations and data checking</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)

## check that sample annot and raw data have same set of sampleIDs:
idintersect = intersect(dimnames(raw)[[1]],dimnames(annot)[[1]])
cat("Samples in annotation omitted from dataset:",file=log,sep='\n',append=TRUE)
suppressWarnings(write.table(setdiff(dimnames(annot)[[1]],dimnames(raw)[[1]]),file=log,sep='\t',append=TRUE))
cat("Samples in dataset omitted from annotation:",file=log,sep='\n',append=TRUE)
suppressWarnings(write.table(setdiff(dimnames(raw)[[1]],dimnames(annot)[[1]]),file=log,sep='\t',append=TRUE))

## re-order sample annotation matrix to match data matrix:
annot = annot[idintersect,]
raw = raw[idintersect,]

### remove the controls from the raw data matrix:
controlnames = c(paste("Pos",c("A","B","C","D","E","F"),sep="_"),paste("Neg",c("A","B","C","D","E","F","G","H"),sep="_"))
raw = raw[,!is.element(dimnames(raw)[[2]],controlnames)]

if(!identical(dimnames(raw)[[1]],dimnames(annot)[[1]])){
  warning("Warning: Sample IDs in sample annotation file don't match sample IDs in expression dataset")
  cat("LOG:Warning: Sample IDs in sample annotation file don't mathc sample IDs in expression dataset",file=log,sep='\n',append=TRUE)
  cat("document.write('<p>Warning: Sample IDs in sample annotation file don't match sample IDs in expression dataset</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
}

### format sample annot:
cat("formatting annotation data frame",file=log,sep='\n',append=TRUE)
suppressWarnings(write.table(head(annot),file=log,sep='\t',append=TRUE))
cat(sampleannot.variables,file=log,sep='\n',append=TRUE)
# make sure no variables that aren't in annot get called later:
deregmodule.arg.is.normal.variable = intersect(sampleannot.variables,deregmodule.arg.is.normal.variable)
deregmodule.arg.covariates = intersect(sampleannot.variables,deregmodule.arg.covariates)
deregmodule.arg.adjust.for = intersect(sampleannot.variables,deregmodule.arg.adjust.for)
DEmodule.arg.predictors = intersect(sampleannot.variables,DEmodule.arg.predictors)
DEmodule.arg.confounders = intersect(sampleannot.variables,DEmodule.arg.confounders)
deregmodule.arg.is.normal.level = as.character(deregmodule.arg.is.normal.level) 
sampleannot.referencelevels = as.character(sampleannot.referencelevels)  

annotdimnames = dimnames(annot)
# 1st, take only variables declared by nSolver:
annot = annot[,sampleannot.variables,drop=FALSE]  ##**
#dimnames(annot)[[1]] = annotdimnames[[1]]
if(length(sampleannot.variables==1)){dimnames(annot)[[2]] = sampleannot.variables}

### check that all sample annot variables have variability
# 1st identify the ones that we need to remove:
vars.to.remove = vars.to.remove.TF = rep(NA,length(sampleannot.variables))
for(i in 1:length(sampleannot.variables))
{
  cat(sampleannot.variables[i],file=log,sep='\n',append=TRUE)
  cat(table(annot[,sampleannot.variables[i]]),file=log,sep='\n',append=TRUE)
  vars.to.remove.TF[i]=FALSE
  vunique=setdiff(unique(annot[,sampleannot.variables[i]]),NA)
  nunique = length(setdiff(unique(annot[,sampleannot.variables[i]]),NA))
  if(nunique<2)
  {
    print(paste("Warning:",sampleannot.variables[i],"has fewer than 2 unique, non-missing values.  It will be removed from all subsequent analyses."))
    cat(paste("LOG:Warning:",sampleannot.variables[i],"has fewer than 2 unique, non-missing values.  It will be removed from all subsequent analyses."),file=log,sep='\n',append=TRUE)
    cat(paste("document.write('<p>Warning: ",sampleannot.variables[i],"has fewer than 2 unique, non-missing values.  It will be removed from all subsequent analyses.</p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
    vars.to.remove[i] = c(sampleannot.variables[i])
    vars.to.remove.TF[i]=TRUE
  }
  if((sampleannot.variabletypes[i]=="categorical")&(max(abs(table(annot[,sampleannot.variables[i]])-1))==0))
  {
    print(paste("Warning: categorical variable",sampleannot.variables[i],"has as many values as observations.  It will be removed from all subsequent analyses."))
    cat(paste("LOG:Warning: categorical variable",sampleannot.variables[i],"has as many values as observations.  It will be removed from all subsequent analyses."),file=log,sep='\n',append=TRUE)
    cat(paste("document.write('<p>Warning: categorical variable",sampleannot.variables[i],"has as many values as observations.  It will be removed from all subsequent analyses.</p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
    vars.to.remove[i] = c(sampleannot.variables[i])
    vars.to.remove.TF[i]=TRUE
  }
  #make sure continuous variables are numbers
  if(sampleannot.variabletypes[i]=="continuous")
  {
    for(j in 1:nunique){
      if(!is.numeric(vunique[[j]])){
        print(paste("Warning: continuous variable",sampleannot.variables[i],"has non-numeric values.  It will be removed from all subsequent analyses."))
        cat(paste("LOG:Warning: continuous variable",sampleannot.variables[i],"has non-numeric values.  It will be removed from all subsequent analyses."),file=log,sep='\n',append=TRUE)
        cat(paste("document.write('<p>Warning: continuous variable",sampleannot.variables[i],"has non-numeric values.  It will be removed from all subsequent analyses.</p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
        vars.to.remove[i] = c(sampleannot.variables[i])
        vars.to.remove.TF[i]=TRUE
        break
      }
    }
  }
    #check for NAs in adjust.for
    if((deregmodule.arg.run)&&(length(deregmodule.arg.adjust.for)>0)&&(is.element(NA,unique(annot[,sampleannot.variables[i]]))))
    {
      if(length(annot[is.na(annot[,sampleannot.variables[i]]),sampleannot.variables[i]])/length(annot[,sampleannot.variables[i]])>0){
        deregmodule.arg.adjust.for=deregmodule.arg.adjust.for[deregmodule.arg.adjust.for != sampleannot.variables[i]]
        print(paste("Warning: variable",sampleannot.variables[i],"contains NA values.  It will not be adjusted for in the pathway scoring analysis."))
        cat(paste("LOG:Warning: variable",sampleannot.variables[i],"contains NA values.  It will not be adjusted for in the pathway scoring analysis."),file=log,sep='\n',append=TRUE)
        cat(paste("document.write('<p>Warning: variable",sampleannot.variables[i],"contains NA values.  It will not be adjusted for in the pathway scoring analysis.</p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
        if(length(deregmodule.arg.adjust.for)==0){deregmodule.arg.adjust.for=NULL}
      }
  }
    #check for NAs in DE
    if((DEmodule.arg.run)&&(length(DEmodule.arg.confounders)>0)&&(is.element(NA,unique(annot[,sampleannot.variables[i]]))))
    {
      if(length(annot[is.na(annot[,sampleannot.variables[i]]),sampleannot.variables[i]])/length(annot[,sampleannot.variables[i]])>0){
        DEmodule.arg.predictors=DEmodule.arg.predictors[DEmodule.arg.predictors != sampleannot.variables[i]]
	DEmodule.arg.confounders=DEmodule.arg.confounders[DEmodule.arg.confounders != sampleannot.variables[i]]
        print(paste("Warning: variable",sampleannot.variables[i],"contains NA values.  It will not be adjusted for in the differential expression analysis."))
        cat(paste("LOG:Warning: variable",sampleannot.variables[i],"contains NA values.  It will not be adjusted for in the differential expression analysis."),file=log,sep='\n',append=TRUE)
        cat(paste("document.write('<p>Warning: variable",sampleannot.variables[i],"contains NA values.  It will not be adjusted for in the differential expression analysis.</p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
        if(length(DEmodule.arg.confounders)==0){DEmodule.arg.confounders=NULL}
	if(length(DEmodule.arg.predictors)==0){DEmodule.arg.confounders=NULL}
      }
  }
  # make sure boolean variables are well-formatted:
  if(sampleannot.variabletypes[i]=="boolean")
  {
    annot[,sampleannot.variables[i]] = toupper(annot[,sampleannot.variables[i]])
    # make sure it's actually boolean
    nonbooleanlevels = setdiff(unique(annot[,sampleannot.variables[i]]),c("FALSE","TRUE",NA))
    if(length(nonbooleanlevels)>0)
    {
      warning(paste("Warning: Boolean variable",sampleannot.variables[i],"has illegal value(s)",nonbooleanlevels,".  It is being dropped from the analysis."))
      cat(paste("LOG: Warning: Boolean variable",sampleannot.variables[i],"has illegal value(s)",nonbooleanlevels,".  It is being dropped from the analysis."),file=log,sep='\n',append=TRUE)
      cat(paste("document.write('<p> Warning: Boolean variable",sampleannot.variables[i],"has illegal value(s)",nonbooleanlevels,".  It is being dropped from the analysis)</p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
      vars.to.remove[i] = c(sampleannot.variables[i])
      vars.to.remove.TF[i]=TRUE
    }
  }
    
}
cat("vars to remove:",file=log,sep='\n',append=TRUE)
suppressWarnings(write.table(vars.to.remove,file=log,sep='\t',append=TRUE))
suppressWarnings(write.table(vars.to.remove.TF,file=log,sep='\t',append=TRUE))
# now remove them:
for(i in length(sampleannot.variables):1)
{
  if(vars.to.remove.TF[i])
  {
    # 1st clean up other arguments using the variable:
    deregmodule.arg.is.normal.variable = setdiff(deregmodule.arg.is.normal.variable,sampleannot.variables[i])
    if(length(deregmodule.arg.is.normal.variable)==0)
    {
      deregmodule.arg.is.normal.level=NULL
      if(deregmodule.arg.method=="Pathifier"){deregmodule.arg.method="none"}
    }
    deregmodule.arg.covariates = setdiff(deregmodule.arg.covariates,sampleannot.variables[i])
    deregmodule.arg.adjust.for = setdiff(deregmodule.arg.adjust.for,sampleannot.variables[i])
    DEmodule.arg.predictors = setdiff(DEmodule.arg.predictors,sampleannot.variables[i])
    DEmodule.arg.confounders = setdiff(DEmodule.arg.confounders,sampleannot.variables[i])
    if(length(DEmodule.arg.predictors)==0)
    {
      DEmodule.arg.run=FALSE
      pathviewmodule.arg.run=FALSE
      gsamodule.arg.run = FALSE
    }
    
    sampleannot.variables=sampleannot.variables[-i]
    sampleannot.variabletypes=sampleannot.variabletypes[-i]
    sampleannot.referencelevels=sampleannot.referencelevels[-i] 
    annot = annot[,-i,drop=FALSE]; ##**
    #dimnames(annot)[[2]] = sampleannot.variables; dimnames(annot)[[1]] = annotdimnames[[1]]
  }
}


cat("sampleannot.variables:",file=log,sep='\n',append=TRUE)
suppressWarnings(write.table(sampleannot.variables,file=log,sep='\t',append=TRUE))
cat("sampleannot.variabletypes:",file=log,sep='\n',append=TRUE)
suppressWarnings(write.table(sampleannot.variabletypes,file=log,sep='\t',append=TRUE))
cat("sampleannot.variabletypes:",file=log,sep='\n',append=TRUE)
suppressWarnings(write.table(sampleannot.variabletypes,file=log,sep='\t',append=TRUE))

# make all variables cat or cont as declared, and set appropriate ref levels when given:

if(length(sampleannot.variables)>0)
{  
  for(i in 1:length(sampleannot.variables))
  {
    suppressWarnings(write.table(sampleannot.variables[i],file=log,sep='\t',append=TRUE))
    suppressWarnings(write.table(sampleannot.variabletypes[i],file=log,sep='\t',append=TRUE))
    
    if(sampleannot.variabletypes[i]=="continuous")
    {
      annot[,sampleannot.variables[i]]=as.numeric(as.vector(annot[,sampleannot.variables[i]]))
      cat("as.numeric(as.vector(annot[,sampleannot.variables[i]]))",file=log,sep='\n',append=TRUE)
      suppressWarnings(write.table(head(annot[,sampleannot.variables[i]]),file=log,sep='\t',append=TRUE))
      if(var(annot[,sampleannot.variables[i]],na.rm=T)==0)
      {
        warning(paste(sampleannot.variables[i],"had zero variance.  It is being dropped from the analysis.")) 
        cat(paste("LOG: Warning: ", sampleannot.variables[i],"had zero variance.  It is being dropped from the analysis."),file=log,sep='\n',append=TRUE)
        cat(paste("document.write('<p>Warning: ", sampleannot.variables[i],"had zero variance.  It is being dropped from the analysis</p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
        annot = annot[,-sampleannot.variables[i],drop=FALSE]  ##**
      }
    }
    if(sampleannot.variabletypes[i]=="categorical")
    {
      annot[,sampleannot.variables[i]]=as.factor(as.character(annot[,sampleannot.variables[i]]))  
      if(!is.na(sampleannot.referencelevels[i]))
      {
        annot[,sampleannot.variables[i]]=relevel(annot[,sampleannot.variables[i]],sampleannot.referencelevels[i])
      }
      if(length(unique(annot[,sampleannot.variables[i]]))==1)
      {
        warning(paste("Warning: ",sampleannot.variables[i],"has only one level.  It is being dropped from the analysis."))
        cat(paste("LOG: Warning: ", sampleannot.variables[i],"has only one level.  It is being dropped from the analysis."),file=log,sep='\n',append=TRUE)
        cat(paste("document.write('<p>Warning: ", sampleannot.variables[i],"has only one level.  It is being dropped from the analysis</p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
        annot = annot[,-sampleannot.variables[i],drop=FALSE]  ##**
      }
    }
    # if boolean: treat is as categorical, w FALSE as the reference:
    if(sampleannot.variabletypes[i]=="boolean")
    {
      # make it uppercase
      annot[,sampleannot.variables[i]] = toupper(annot[,sampleannot.variables[i]])
      sampleannot.variabletypes[i]="categorical"
      annot[,sampleannot.variables[i]]=as.factor(as.character(annot[,sampleannot.variables[i]]))
      annot[,sampleannot.variables[i]]=relevel(annot[,sampleannot.variables[i]],"FALSE")
      if(length(unique(annot[,sampleannot.variables[i]]))==1)
      {
        warning(paste("Warning: ",sampleannot.variables[i],"has only one level.  It is being dropped from the analysis."))
        cat(paste("LOG: Warning: ", sampleannot.variables[i],"has only one level.  It is being dropped from the analysis."),file=log,sep='\n',append=TRUE)
        cat(paste("document.write('<p>Warning: ", sampleannot.variables[i],"has only one level.  It is being dropped from the analysis</p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
        annot = annot[,-sampleannot.variables[i],drop=FALSE]  ##**
      }
    }
  }
}

cat("dim(raw)=",file=log,sep='\n',append=TRUE)
cat(dim(raw),file=log,sep='\n',append=TRUE)
cat("dim(annot)=",file=log,sep='\n',append=TRUE)
cat(dim(annot),file=log,sep='\n',append=TRUE)
cat("dim(gannot)=",file=log,sep='\n',append=TRUE)
cat(dim(gannot),file=log,sep='\n',append=TRUE)

# remove all but endogenous and housekeeping genes from gannot:
cat("top of gannot",file=log,sep='\n',append=TRUE)
suppressWarnings(write.table(gannot[1:5,1:5],file=log,sep='\t',append=TRUE))

# final alignment of raw, annot, gannot:
intersect.sample = intersect(dimnames(raw)[[1]],dimnames(annot)[[1]])
intersect.gene = intersect(dimnames(raw)[[2]],dimnames(gannot)[[1]])
raw = raw[intersect.sample,intersect.gene]
annot = annot[intersect.sample,,drop=FALSE]
gannot = gannot[intersect.gene,]

## re-order data matrix to match gene annotation matrix:
#raw = raw[,setdiff(match(as.vector(dimnames(gannot)[[1]]),as.vector(dimnames(raw)[[2]])),NA)]
cat("top of raw",file=log,sep='\n',append=TRUE)
suppressWarnings(write.table(raw[1:5,1:5],file=log,sep='\t',append=TRUE))
cat("dim(raw)=",file=log,sep='\n',append=TRUE)
cat(dim(raw),file=log,sep='\n',append=TRUE)

if(!identical(as.vector(dimnames(raw)[[2]]),as.vector(dimnames(gannot)[[1]]))){
  warning("Warning: Gene names in gene annotation file don't match gene names in expression dataset")
  cat("LOG: Warning: Gene names in gene annotation file don't match gene names in expression dataset",file=log,sep='\n',append=TRUE)
  cat("document.write('<p>Warning: Gene names in gene annotation file don't match gene names in expresssion dataset</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)


}
###### parse gene annotation:
## split data into endog and hk data:
e1 = raw[,is.element(dimnames(raw)[[2]],dimnames(gannot)[[1]][tolower(gannot$Class.Name)=="endogenous"])]
e1 = as.matrix(e1)
hk1 = raw[,is.element(dimnames(raw)[[2]],dimnames(gannot)[[1]][tolower(gannot$Class.Name)=="housekeeping"])]
hk1 = as.matrix(hk1)
rawdata = cbind(e1,hk1)
cat("dim(e1):",file=log,sep='\n',append=TRUE)
cat(dim(e1),file=log,sep='\n',append=TRUE)
cat("dim(hk1):",file=log,sep='\n',append=TRUE)
cat(dim(hk1),file=log,sep='\n',append=TRUE)

print ("Finished annotations and data checking")
cat("LOG:Finished annotations and data checking",file=log,sep='\n\n',append=TRUE)
cat("document.write('<p>Finished annotations and data checking</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)

##### assign colors to all variables:
# colors to use:
codecols2 = c("chartreuse3","darkgoldenrod2","deepskyblue2","firebrick2","darkorchid1","tan","slateblue3","forestgreen","coral","gray48",
              "aquamarine","bisque2","blue3","blueviolet","brown2","burlywood3","darkgoldenrod4","cyan3","cornflowerblue",
              "chocolate2","chartreuse1","darkorange","darkorchid4","darkred","darkslateblue","darkturquoise","deeppink1",
              "deeppink4","gray7","gold1","dodgerblue1")
codecols2 = c(codecols2,colors())
contcols = list()
contcols[[1]] = c("deepskyblue2","grey","firebrick2")
contcols[[2]] = c("forestgreen","grey","darkorange")
contcols[[3]] = c("chartreuse","grey","gold1")
contcols[[4]] = c("darkorchid1","grey","darkgoldenrod2")
contcols[[5]] = c("darkslateblue","grey","chocolate2")
for(i in 6:100){contcols[[i]]=contcols[[i-5]]}

if(dim(as.data.frame(annot))[2]>0)
{
  # matrix for coloring each observation by each variable:
  annotcols = matrix("white",dim(as.data.frame(annot))[1],dim(as.data.frame(annot))[2])
  # list of the unique colors for each annotation
  annotcols2 = list()
  dimnames(annotcols)[[1]] = dimnames(annot)[[1]]
  dimnames(annotcols)[[2]] = dimnames(annot)[[2]]
  colsused = 0; contcolsused=0
  for(i in dim(as.data.frame(annot))[2]:1)
  {
    # ID type of covariate
    if(sampleannot.variabletypes[i] == "continuous")
    {
      annotcols[,i] = colorRampPalette(contcols[[1+contcolsused]])( 101 )[round((annot[,i]-min(annot[,i]))/(max(annot[,i])-min(annot[,i]))*100+1,0)]
      
      annotcols2[[i]] = contcols[[1+contcolsused]]
      contcolsused = contcolsused+1
    }
    if(sampleannot.variabletypes[i] == "categorical")
    {
      annotcols[,i] = codecols2[(colsused+1):length(codecols2)][annot[,i]]
      annotcols2[[i]] = codecols2[(colsused+1):length(codecols2)][1:length(unique(annot[,i]))]
      colsused = length(unique(annot[,i]))+colsused
      names(annotcols2[[i]]) = levels(annot[,i])  
    }
  }
}

##### arguments to be hard-coded in:
deregmodule.arg.pathways = c("Notch","Wnt","HH","ChromMod","TXmisReg","DNARepair","TGFB","MAPK",    
                             "STAT","PI3K","RAS","Apop","CC")
deregmodule.arg.plot.cormatrix = TRUE
deregmodule.arg.draw.networks = TRUE
deregmodule.arg.pairs.plot.of.scores = TRUE
deregmodule.arg.PC.pairs.plot.of.scores = TRUE
deregmodule.arg.plot.against.covariates = (length(deregmodule.arg.covariates)>0)

pathviewmodule.arg.keggIDs = c("04330","04310","04340","04630","04151","04014","04210","04110","04350","04010")
pathviewmodule.arg.local.xml.location = paste(path.pipeline.base,"//KEGG_xml_files",sep="")

#### normalize data:

cat("running normalization module",file=log,sep='\n',append=TRUE)
cat(timestamp(),file=log,sep='\n',append=TRUE)
if(!normmodule.arg.run){normmodule.arg.method="none"}
normresult = normalizerawdata(raw=as.matrix(rawdata),method=normmodule.arg.method,
                            choose.method = "geNorm",
                            n.HKs = normmodule.arg.n.HKs,auto.HKs = normmodule.arg.auto.HKs,
                            codeset.HKs = gannot$is.HK)
e = normresult$normalized

#### run QC:
suppressWarnings(write.table(head(as.data.frame(annot)),file=log,sep='\t',append=TRUE))
if(TRUE)
{
cat("running QC module",file=log,sep='\n',append=TRUE)
#HTML NOTE - the HTML for the QC is embedded in the other R-file
cat(timestamp(),file=log,sep='\n',append=TRUE)
gene.set.matrix = gannot[,deregmodule.arg.pathways]; dimnames(gene.set.matrix)[[1]] = dimnames(gannot)[[1]]
  QCresult = runQC(normalized=e,HKnormalized=normresult$HKnormalized,HKs=normresult$HKs,
                   covariates=as.data.frame(annot),
                   covariates.type=sampleannot.variabletypes,
                 gene.sets=gene.set.matrix, gene.set.names=pathwaynames,annotcols=annotcols,annotcols2=annotcols2,pruneflag=pruneflag)   
}
#### fit pathway scores:
gene.set.matrix=NULL
if(deregmodule.arg.run)
{
  cat("running pathway deregulation scoring module",file=log,sep='\n',append=TRUE)
  cat(timestamp(),file=log,sep='\n',append=TRUE)
  suppressWarnings(write.table(head(annot),file=log,sep='\t',append=TRUE))
  gene.set.matrix = gannot[,deregmodule.arg.pathways]; dimnames(gene.set.matrix)[[1]] = dimnames(gannot)[[1]]

  is.normal.temp = NULL
  if((length(deregmodule.arg.is.normal.variable)==1)&(length(deregmodule.arg.is.normal.level)==1))
  {
    is.normal.temp=relevel(annot[,deregmodule.arg.is.normal.variable],deregmodule.arg.is.normal.level)
  }
  # identify genes that are all in the BG - should be omitted from analysis:
  gene.indices.to.omit = NULL
  if(deregmodule.arg.method != "Pre-defined")
  {
    gene.names.to.omit = dimnames(raw)[[2]][apply(raw,2,max)<20]
    gene.indices.to.omit = is.element(dimnames(e)[[2]],gene.names.to.omit)
  }
  
  # format the covariates matrix:
  covariates.for.pathscoring=annot[,deregmodule.arg.covariates,drop=FALSE]; ##**
  if(dim(covariates.for.pathscoring)[2]>0){dimnames(covariates.for.pathscoring)[[2]] = deregmodule.arg.covariates}
  deregresults = build.pathway.scores(normalized=e,method=deregmodule.arg.method,
                                      gene.sets=gene.set.matrix,
                                      gene.set.names=pathwaynames,
                                      #is.normal = relevel(annot[,deregmodule.arg.is.normal],deregmodule.arg.is.normal.level), 
                                      is.normal = is.normal.temp, 
                                      path.to.predefined.scores=predefinedmodule.arg.path.to.weights,
                                      plot.cormatrix = deregmodule.arg.plot.cormatrix,
                                      draw.networks=deregmodule.arg.draw.networks,
                                      pairs.plot.of.scores=deregmodule.arg.pairs.plot.of.scores,
                                      PC.pairs.plot.of.scores=TRUE,
                                      plot.against.covariates=TRUE,
                                      covariates=covariates.for.pathscoring,
                                      covariate.type=sampleannot.variabletypes[deregmodule.arg.covariates],
                                      adjust.for = annot[,deregmodule.arg.adjust.for,drop=FALSE], ##**
                                      genes.to.omit = gene.indices.to.omit,
                                      annotcols=as.matrix(annotcols[,deregmodule.arg.covariates]),
                                      annotcols2=annotcols2[match(deregmodule.arg.covariates,dimnames(annot)[[2]])])
  # if no results were returned for some reason, set deregmodule.arg.run to FALSE: 
  if(length(deregresults$scores)==0)
  {
    deregmodule.arg.run=FALSE
  }
}



#### next: DE analyses:

if(length(DEmodule.arg.predictors)==0){DEmodule.arg.run=FALSE}
if(DEmodule.arg.run)
{
  gene.set.matrix = gannot[,deregmodule.arg.pathways]; dimnames(gene.set.matrix)[[1]] = dimnames(gannot)[[1]]  
  
  cat("running DE module",file=log,sep='\n\n',append=TRUE)
  cat(timestamp(),file=log,sep='\n',append=TRUE)
  

  cat("predictors:",file=log,sep='\n',append=TRUE)
  suppressMessages(suppressWarnings(write.table(DEmodule.arg.predictors,file=log,sep='\t',append=TRUE)))
  cat("confounders:",file=log,sep='\n',append=TRUE)
  suppressMessages(suppressWarnings(write.table(DEmodule.arg.confounders,file=log,sep='\t',append=TRUE)))
  cat("predictor var types:",file=log,sep='\n',append=TRUE)
  suppressMessages(suppressWarnings(write.table(sampleannot.variabletypes[DEmodule.arg.predictors],file=log,sep='\t',append=TRUE)))
  cat("confounder var types:",file=log,sep='\n',append=TRUE)
  suppressMessages(suppressWarnings(write.table(sampleannot.variabletypes[DEmodule.arg.confounders],file=log,sep='\t',append=TRUE)))
  
  DEresults = run.DE.analyses(normalized=e,predictors=DEmodule.arg.predictors,
                              confounders=DEmodule.arg.confounders,
                              annot=annot,
                              predictor.var.type=sampleannot.variabletypes[DEmodule.arg.predictors],
                              confounder.var.type=sampleannot.variabletypes[DEmodule.arg.confounders],
                              pval.adjustment = DEmodule.arg.pval.adjustment,gene.sets=gene.set.matrix)  
}

### run pathview:  
if(!DEmodule.arg.run){pathviewmodule.arg.run=FALSE}
if(!testCon()){
  pathviewmodule.arg.run=FALSE
  warning("No internet connection - Pathview will not be run")
  cat("LOG: Warning: No internet connection - Pathview will not be run",file=log,sep='\n',append=TRUE)
  cat("document.write('<p>Warning: No internet connection - Pathview will not be run</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
}
if(pathviewmodule.arg.run)
{
  cat("running Pathview module",file=log,sep='\n\n',append=TRUE)
  cat(timestamp(),file=log,sep='\n',append=TRUE)
  # clean the custom IDs to make them parse right: rule: 5 digits, all numbers.
  #<----------------------------------
  pathview.results = run.pathview(DEresult=DEresults, 
                                  keggIDs=c(pathviewmodule.arg.keggIDs,as.character(pathviewmodule.arg.keggIDs.custom)),
                                  colorscheme = pathviewmodule.arg.view.colorscheme,
                                  pvalthresh=pathviewmodule.arg.pvalthres,
                                  view=c("KEGG"),
                                  kegg.dir = pathviewmodule.arg.local.xml.location,
						  predictors=DEmodule.arg.predictors)

  setwd(analysis.path)
}

### run gene set analysis:
if(!DEmodule.arg.run){gsamodule.arg.run=FALSE}
if(gsamodule.arg.run)
{
  cat("running gene set analysis module",file=log,sep='\n\n',append=TRUE)
  cat(timestamp(),file=log,sep='\n',append=TRUE)
  gsaresults = run.gene.set.analyses(DEresults,gene.sets=gene.set.matrix,
                                     gene.set.names=deregmodule.arg.pathways,
                                     pval.adjustment=DEmodule.arg.pval.adjustment,
							predictors=DEmodule.arg.predictors)
}
  


### rank pathways by how interesting they are (for each covariate):
# rank by global sig score
# rank by association btw pathway score and covariate
# plot both associations against each other

## 1st: run pathscores through DE and GSA moduled:
if(gsamodule.arg.run&deregmodule.arg.run)
{
  print("Running pathway significance analysis - DE module will re-run")
  cat("LOG: Running pathway significance analysis - DE module will re-run",file=log,sep='\n',append=TRUE)
  cat("document.write('<p>Running pathway significance analysis - DE module will re-run</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
  cat(timestamp(),file=log,sep='\n',append=TRUE)
  DEresults.pathscore = run.DE.analyses(normalized=deregresults$scores,predictors=DEmodule.arg.predictors,
                                        confounders=DEmodule.arg.confounders,annot=annot,
                                        predictor.var.type=sampleannot.variabletypes[DEmodule.arg.predictors],
                                        #predictor.cat.reference=DEmodule.arg.predictor.cat.reference,
                                        confounder.var.type=sampleannot.variabletypes[DEmodule.arg.confounders],
                                        pval.adjustment = DEmodule.arg.pval.adjustment,gene.sets=gene.set.matrix,saveresults=FALSE)  

  summarize.pathscores.and.DE(deregresults=deregresults,gsaresults=gsaresults,DEresults.pathscore=DEresults.pathscore,predictors=DEmodule.arg.predictors) 
}
  

#---HTML report code injection---#

#create top menu
cat("document.write('<li class=\"tab-title active\"><a href=\"#panel2-6\">QC&nbsp;</a></li>');\n",file=paste(path.inc,"//top_menu.js",sep=""),append=TRUE)
if(deregmodule.arg.run)
{
	cat("document.write('<li class=\"tab-title\"><a href=\"#panel2-1\">Pathway Score</a></li>');\n",file=paste(path.inc,"//top_menu.js",sep=""),append=TRUE)
	if(DEmodule.arg.run&gsamodule.arg.run)
	{
		cat("document.write('<li class=\"tab-title\"><a href=\"#panel2-8\">Pathway Sig Plots</a></li>')\n",file=paste(path.inc,"//top_menu.js",sep=""),append=TRUE)
	}
}
if(gsamodule.arg.run)
{
	cat("document.write('<li class=\"tab-title\"><a href=\"#panel2-3\">GSA</a></li>');\n",file=paste(path.inc,"//top_menu.js",sep=""),append=TRUE)

}
if(pathviewmodule.arg.run)
{
	cat("document.write('<li class=\"tab-title\"><a href=\"#panel2-4\">PathView</a></li>');\n",file=paste(path.inc,"//top_menu.js",sep=""),append=TRUE)

}
if(DEmodule.arg.run)
{
	cat("document.write('<li class=\"tab-title\"><a href=\"#panel2-2\">Diff Expr</a></li>');\n",file=paste(path.inc,"//top_menu.js",sep=""),append=TRUE)
}
if(normmodule.arg.run)
{
	cat("document.write('<li class=\"tab-title\"><a href=\"#panel2-5\">Normalization</a></li>');\n",file=paste(path.inc,"//top_menu.js",sep=""),append=TRUE)
}
cat("document.write('<li class=\"tab-title \"><a href=\"#panel2-7\">Summary</a></li>');\n",file=paste(path.inc,"//top_menu.js",sep=""),append=TRUE)


#--- END HTML report code injection---#





######################## 
#### copy all results to a findable directory:
cat("making results file directories",file=log,sep='\n\n',append=TRUE)
# append the date and time to the analysis name for the purpose of copying files:
advanced.analysis.name.for.file.copy = paste(advanced.analysis.name,gsub(":","-",substr(Sys.time(),1,16)))
results.copydir = paste(path.results.copy,"//",advanced.analysis.name.for.file.copy,"//",sep="")

# make directories:
dir.create(results.copydir)
path.to.normalization.results2 = paste(results.copydir,"Normalization",sep=""); dir.create(path.to.normalization.results2)
path.to.QC.results2 = paste(results.copydir,"QC",sep=""); dir.create(path.to.QC.results2)
path.to.pathway.scores2 = paste(results.copydir,"Pathway scoring",sep=""); dir.create(path.to.pathway.scores2)
path.to.DE.results2 = paste(results.copydir,"DE",sep=""); dir.create(path.to.DE.results2)
path.to.pathview.results2 = paste(results.copydir,"Pathview",sep=""); dir.create(path.to.pathview.results2)
path.to.GSA.results2 = paste(results.copydir,"Gene set analysis",sep=""); dir.create(path.to.GSA.results2)
path.to.csvs2 = paste(results.copydir,"csvs",sep=""); dir.create(path.to.csvs2)
# and copy results:
# from main results dir:
cat("copying results files",file=log,sep='\n\n',append=TRUE)
setwd(path.results)
tfiles = dir()
for(j in 1:length(tfiles))
{
  invisible(file.copy(from = tfiles[j],to = paste(results.copydir,tfiles[j],sep="")))
}
# from results subdirs:
subdirs = c("Normalization","QC","Pathway scoring","DE","Pathview","Gene set analysis","csvs")
for(i in 1:length(subdirs))
{
  setwd(paste(path.results,"//",subdirs[i],sep=""))
  tfiles = dir()
  for(j in 1:length(tfiles))
  {
    #invisible(file.copy(from = tfiles[j],to = paste(results.copydir,"//",subdirs[i],sep="")))   #<---- 11-7: deleted in favor of the below line by PD.
    invisible(file.copy(from = tfiles[j],to = paste(results.copydir,subdirs[i],sep="")))  
  }
}

}#end runeverything function

#create directories
analysis.path = getwd()
log=paste(getwd(),"//log_file.txt",sep="")
path.resources = paste(getwd(),"//resources",sep="");dir.create(path.resources)
path.js = paste(path.resources,"//js",sep="");dir.create(path.js)
path.css = paste(path.resources,"//css",sep="");dir.create(path.css)
path.img = paste(path.resources,"//img",sep="");dir.create(path.img)
path.inc = paste(path.resources,"//inc",sep="");dir.create(path.inc)
path.results = paste(getwd(),"//results",sep="");dir.create(path.results)

path.to.normalization.results = paste(path.results,"//Normalization",sep=""); dir.create(path.to.normalization.results)
path.to.QC.results = paste(path.results,"//QC",sep=""); dir.create(path.to.QC.results)
path.to.pathway.scores = paste(path.results,"//Pathway scoring",sep=""); dir.create(path.to.pathway.scores)
path.to.DE.results = paste(path.results,"//DE",sep=""); dir.create(path.to.DE.results)
path.to.pathview.results = paste(path.results,"//Pathview",sep=""); dir.create(path.to.pathview.results)
path.to.GSA.results = paste(path.results,"//Gene set analysis",sep=""); dir.create(path.to.GSA.results)
path.to.csvs = paste(path.results,"//csvs",sep=""); dir.create(path.to.csvs)
# arguments for processing gene annotation: columns of gene annotation file: gene name, HK status, pathway memberships
pathwaynames = c("Notch","Wnt","HH","ChromMod","TXmisReg","DNARepair","TGFB","MAPK","STAT","PI3K","RAS","Apop","CC")



print("Starting setup")
cat("LOG:Starting setup",file=log,sep='\n')
cat("document.write('<p>Starting setup</p>');", file=paste(path.inc,"//status.js",sep=""))

#####################
source("pancancer_analysis_module_048.r")

cat("loaded source",file=log,sep='\n\n',append=TRUE)
#####################

# read the arguments file:
cat("loading arguments file",file=log,sep='\n')
source("arguments.r") 
plottypearg = unique(c("png",plottypearg))


result <- try(runanalysis())
warnings()
if(class(result) == "try-error"){
  print("Analysis could not be completed")
  cat("LOG: Analysis could not be completed",file=log,sep='\n',append=TRUE)
  cat(result,file=log,sep='\n',append=TRUE)
  cat("document.write('<p>Analysis could not be completed</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
} else{
  print("Analysis is complete")
  cat("LOG:Analysis is complete",file=log,sep='\n',append=TRUE)
  cat("document.write('<p>Analysis is complete</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
  cat("window.location = \"report.html\";", file=paste(path.inc,"//redirect.js",sep=""),append=TRUE)
}
