
# colors to use in plots:
codecols = c("chartreuse3","darkgoldenrod1","darkgoldenrod4")
codecols2 = c("chartreuse2","darkgoldenrod3","deepskyblue2","firebrick2","darkorchid1","tan","slateblue3","forestgreen","coral","gray48",
              "aquamarine","bisque2","blue3","blueviolet","brown2","burlywood3","darkgoldenrod4","cyan3","cornflowerblue",
              "chocolate2","chartreuse1","darkorange","darkorchid4","darkred","darkslateblue","darkturquoise","deeppink1",
              "deeppink4","gray7","gold1","dodgerblue1")
codecols2 = c(codecols2,colors())


downloadpackages <- function(location){
  
  site = "http://cran.us.r-project.org"  
  #checks if package is installed; if not, function downloads the library from the web
  NSL<-cbind(read.table(paste(location,'//pckg_v_list.csv',sep=''), header = FALSE, sep = ','), Installed = 0)
  userL<-subset(installed.packages(),select = c(Package, Version))
  
  for (i in seq(1,length(NSL[,1]))){
    for (j in seq(1,length(userL[,1]))){
      if (NSL[i,1]==userL[j,1]){
        NSL[i,4]=1
        cat(paste(NSL[i,1]," is already installed.",sep=''),file=log,sep='\n',append=TRUE)
        break
      }
    }
    if (NSL[i,4]==0){
      if(!testCon()){
        cat("document.write('<p>Cannot continue without an internet connection - analysis stopped</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
        stop("Packages cannot be downloaded without internet connection - analysis stopped")
      }
  source("http://bioconductor.org/biocLite.R")
      if (NSL[i,3]=="BC"){
        suppressMessages(suppressWarnings(biocLite(paste(NSL[i,1],sep=''))))
        cat(paste(NSL[i,1]," is now installed.",sep=''),file=log,sep='\n',append=TRUE)
      }
      if (NSL[i,3]=="CRAN"){
        suppressMessages(install.packages(paste(NSL[i,1],sep=''), repos = site, quiet = TRUE))
        cat(paste(NSL[i,1]," is now installed.",sep=''),file=log,sep='\n',append=TRUE)
      }
    }
    if(i%%5==0||i%%length(NSL[,1])==0){cat(paste("document.write('<p>Loaded ",i,"/",length(NSL[,1])," libraries</p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE)}
  }
}

requirelibraries = function()
{
  suppressMessages(require(NormqPCR))
  suppressMessages(require(gplots))
  suppressMessages(require(pathview))
  suppressMessages(require(pathifier))
  suppressMessages(require(xtable))
}


#### function to load all the required packages:
getpackages <- function(location){
  #checks if package is installed; if not, function installs zip package from the input location
  NSL<-cbind(read.table(paste(location,'//pckg_v_list.csv',sep=''), header = FALSE, sep = ','), Installed = 0)
  userL<-subset(installed.packages(),select = c(Package, Version))
  
  for (i in seq(1,length(NSL[,1]))){
    for (j in seq(1,length(userL[,1]))){
      if (NSL[i,1]==userL[j,1]&&NSL[i,2]==userL[j,2]){
        NSL[i,4]=1
        cat(paste(NSL[i,1]," is already installed.",sep=''),file=log,sep='\n',append=TRUE)
        break
      }
    }
    if (NSL[i,4]==0){
      suppressMessages(install.packages(paste(location,'//',NSL[i,1],'.zip',sep=''), repos = NULL, quiet = TRUE))
      cat(paste(NSL[i,1]," is now installed.",sep=''),file=log,sep='\n',append=TRUE)
    }
  if(i%%5==0||i%%length(NSL[,1])==0){cat(paste("document.write('<p>Loaded ",i,"/",length(NSL[,1])," libraries</p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE)}
  }
}

### function to write a txt file summarizing the arguments:
summarize.arguments.file = function(outfile)
{
  # basic high-level choices:
  cat(paste("document.write('<p>Analysis details:</p>');\n"),file=outfile,append=TRUE)
  cat(paste("document.write('<p>Analysis name: ",advanced.analysis.name,"</p>');\n"),file=outfile,append=TRUE)
  cat(paste("document.write('<p>Location of results: ",path.results.copy,"</p>');\n"),file=outfile,append=TRUE)
  cat(paste("document.write('<p>Type of data:",data.type,"</p>');\n"),file=outfile,append=TRUE)
  cat(paste("document.write('<p>File type of plots: ",plottypearg,"</p>');\n"),file=outfile,append=TRUE)
  
  cat(paste("document.write('<p>Low count threshold details:</p>');\n"),file=outfile,append=TRUE)
  cat(paste("document.write('<p>Remove Genes Below Specified Threshold:",prunegenes.run,"</p>');\n"),file=outfile,append=TRUE)
  cat(paste("document.write('<p>Threshold count value: ",prunegenes.bg.level ,"</p>');\n"),file=outfile,append=TRUE)
  cat(paste("document.write('<p>Remove genes below the threshold at frequency greater than:",prunegenes.missing.freq ,"</p>');\n"),file=outfile,append=TRUE)
  
  # annotation details:
  cat(paste("document.write('<p>Sample annotation details:</p>');\n"),file=outfile,append=TRUE)
  cat(paste("document.write('<p>Unique sample identifier: ",sample.annotation.id.column,"</p>');\n"),file=outfile,append=TRUE)
  for(i in 1:length(sampleannot.variables))
  {
    cat(paste("document.write('<p>Covariate",i,": ",sampleannot.variables[i],"; Variable type: ",sampleannot.variabletypes[i],"; Reference level: ",sampleannot.referencelevels[i],"</p>');\n",sep=""),file=outfile,append=TRUE)
  }
  
  # normalization details:
  cat(paste("document.write('<p>Normalization details:</p>');\n"),file=outfile,append=TRUE)
  cat(paste("document.write('<p>Perform normalization: ",normmodule.arg.run,"</p>');\n"),file=outfile,append=TRUE)
  cat(paste("document.write('<p>Number of housekeepers: ",normmodule.arg.n.HKs,"</p>');\n"),file=outfile,append=TRUE)
  cat(paste("document.write('<p>Auto-select number of housekeepers: ",normmodule.arg.auto.HKs,"</p>');\n"),file=outfile,append=TRUE)
  
  # pathway scoring details:
  cat(paste("document.write('<p>Pathway scoring details:</p>');\n"),file=outfile,append=TRUE)
  cat(paste("document.write('<p>Perform pathway scoring: ",deregmodule.arg.run,"</p>');\n"),file=outfile,append=TRUE)
  cat(paste("document.write('<p>Pathway scoring method: ",deregmodule.arg.method,"</p>');\n"),file=outfile,append=TRUE)
  cat(paste("document.write('<p>Pathway scoring baseline variable: ",deregmodule.arg.is.normal.variable,"using",deregmodule.arg.is.normal.level,"</p>');\n"),file=outfile,append=TRUE)
  cat(paste("document.write('<p>Plot pathway scores vs.: ",paste(deregmodule.arg.covariates,sep="; "),"</p>');\n"),file=outfile,append=TRUE)
  cat(paste("document.write('<p>Adjust pathway scores for: ",paste(deregmodule.arg.adjust.for,sep="; "),"</p>');\n"),file=outfile,append=TRUE)
  
  # DE details:
  cat(paste("document.write('<p>Differential expression analysis details:</p>');\n"),file=outfile,append=TRUE)
  cat(paste("document.write('<p>Perform differential expression testing:",deregmodule.arg.run,"</p>');\n"),file=outfile,append=TRUE)
  cat(paste("document.write('<p>Predictors: ",paste(DEmodule.arg.predictors, sep="; "),"</p>');\n"),file=outfile,append=TRUE)
  cat(paste("document.write('<p>Confounders: ",paste(DEmodule.arg.confounders,sep="; "),"</p>');\n"),file=outfile,append=TRUE)
  cat(paste("document.write('<p>P-value adjustment:",DEmodule.arg.pval.adjustment,"</p>');\n"),file=outfile,append=TRUE)
  cat(paste("document.write('<p>Run gene set analysis:",gsamodule.arg.run,"</p>');\n"),file=outfile,append=TRUE)
  
  # Pathview details:
  cat(paste("document.write('<p>Pathview details:</p>');\n"),file=outfile,append=TRUE)
  cat(paste("document.write('<p>Display results using Pathview:",pathviewmodule.arg.run,"</p>');\n"),file=outfile,append=TRUE)
  pathviewmodule.arg.view.colorscheme2 = pathviewmodule.arg.view.colorscheme
  if(pathviewmodule.arg.view.colorscheme=="DE"){pathviewmodule.arg.view.colorscheme2 = "Fold-change"}
  if(pathviewmodule.arg.view.colorscheme=="tstat"){pathviewmodule.arg.view.colorscheme2 = "t-statistic"}
  cat(paste("document.write('<p>Color Pathview pathview plots by:",pathviewmodule.arg.view.colorscheme2,"</p>');\n"),file=outfile,append=TRUE)
  cat(paste("document.write('<p>P-value threshold:",pathviewmodule.arg.pvalthres,"</p>');\n"),file=outfile,append=TRUE)
  cat(paste("document.write('<p>Additional KEGG IDs: ",paste(pathviewmodule.arg.keggIDs.custom,sep="; "),"</p>');\n"),file=outfile,append=TRUE)
}

### function to draw a plot using any desired output type
drawplot = function(filename,type = c("png","pdf","tiff","jpg","bmp"),width = 1, height = 1,heatmapres=FALSE)
{
  # modify filename to remove illegal characters:
  illegalcharacters = c("<",">",":","\"","\\?","\\|","\\*")  #
  # only work on the later part of the filename after the "C://":
  filename1 = substr(filename,1,4)
  filename2 = substr(filename,5,length(strsplit(filename,split="")[[1]]))
  for(i in 1:length(illegalcharacters))
  {
    filename2 = gsub(illegalcharacters[i],"_",filename2);  #print(illegalcharacters[i]); print(filename2)
  }
  filename = paste(filename1,filename2,sep="")
  # modify resolution for heatmaps:
  res = NA; pointsize = 12
  if(heatmapres){res = 200;pointsize = 7}
  
  # run plotting function:
  if(type=="png")
  {
    png(filename=paste(filename,type,sep="."),width=480*width,height=480*height,res=res,pointsize=pointsize)
  }
  if(type=="jpg")
  {
    jpeg(filename=paste(filename,type,sep="."),width=480*width,height=480*height)
  }
  if(type=="bmp")
  {
    bmp(filename=paste(filename,type,sep="."),width=480*width,height=480*height)
  }
  if(type=="pdf")
  {
    pdf(file=paste(filename,type,sep="."),width=7*width,height=7*height)
  }
  if(type=="tiff")
  {
    tiff(filename=paste(filename,type,sep="."),width=480*width,height=480*height)
  }
  return(filename) 
}
##### function to correct an illegal filename:
correct.filename = function(filename)
{
  # modify filename to remove illegal characters:
  illegalcharacters = c("<",">",":","\"","\\?","\\|","\\*")  
  # only work on the later part of the filename after the "C://":
  filename1 = substr(filename,1,4)
  filename2 = substr(filename,5,length(strsplit(filename,split="")[[1]]))
  for(i in 1:length(illegalcharacters))
  {
    filename2 = gsub(illegalcharacters[i],"_",filename2);  #print(illegalcharacters[i]); print(filename2)
  }
  filename = paste(filename1,filename2,sep="")
  return(filename)
}

###########################################################################
###########################################################################
#               Normalization
###########################################################################
###########################################################################

#### function to normalize the raw data given a list of HKs:
normalize.given.HKs = function(raw,HKdata,HKs)
{   
  # use only the selected HKs:
  HKdata.selected = HKdata[,is.element(dimnames(HKdata)[[2]],HKs)]
  # get mean of HKs
  HKmeans = apply(HKdata.selected,1,mean)
  # apply norm factors:
  norm.factors = HKmeans-mean(HKmeans)
  normalized = raw-norm.factors
  HKnormalized = HKdata-norm.factors
  out = list(normalized=normalized,HKnormalized=HKnormalized,norm.factors=norm.factors)
  return(out)
}

normalizerawdata = function(raw,method=c("choose HKs","specific HKs","none"),choose.method = c("geNorm"),
                            n.HKs = 10,auto.HKs,codeset.HKs)
{
print("Starting normalization")
cat("LOG:Starting normalization",file=log,sep='\n\n',append=TRUE)
cat("document.write('<p>Starting normalization</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
#--- START HTML report code injection---#
# create normalization include file - pretty static as there's not much here...embedding it in case there is a need to edit it
cat("document.write('		<div class=\"large-12 columns\">');\n",file=paste(path.inc,"//panel2_5.js",sep=""),append=TRUE)
cat("document.write('		  	<ul class=\"small-block-grid-2\">');\n",file=paste(path.inc,"//panel2_5.js",sep=""),append=TRUE)
cat("document.write('		  	  <li>Pairwise Variance during HK Selection<br><img src=\"results/Normalization/HK selection details - pairwise variance.png\" ></img></li>');\n",file=paste(path.inc,"//panel2_5.js",sep=""),append=TRUE)
cat("document.write('		  	  <li>Expression - Before and After Normalization<br><img src=\"results/Normalization/total expression before and after normalization.png\"></img></li>');\n",file=paste(path.inc,"//panel2_5.js",sep=""),append=TRUE)
cat("document.write('		  	</ul>');\n",file=paste(path.inc,"//panel2_5.js",sep=""),append=TRUE)
#--- END HTML report code injection---#





  codeset.genes = dimnames(raw)[[2]]
  
  ### basic processing: threshold and log-transform:
  raw = replace(raw,raw<1,1)
  raw = log2(raw)
  ### split data into HKs and informative genes:
  # (1st condition the codeset.HKs variable)
  codeset.HKs[is.na(codeset.HKs)]=0
  cat("table(codeset.HKs):",file=log,sep='\n',append=TRUE)
  cat(table(codeset.HKs),file=log,sep='\n',append=TRUE)
  cat("sum(codeset.HKs):",file=log,sep='\n',append=TRUE)
  cat(sum(codeset.HKs),file=log,sep='\n',append=TRUE)
  suppressWarnings(write.table(table(codeset.HKs==1),file=log,sep='\t',append=TRUE))
  suppressWarnings(write.table(table(codeset.HKs),file=log,sep='\t',append=TRUE))
  cat("dim(raw):",file=log,sep='\t',append=TRUE)
  cat(dim(raw),file=log,sep='\t',append=TRUE)
  HKdata = raw[,codeset.HKs==1]
  raw = raw[,codeset.HKs==0]
  ### if it's already normalized data, do nothing:
  if(method=="none")
  {
    warning("Warning: No normalization will be performed - assuming the input data is already normalized")
    cat("LOG: Warning: No normalization will be performed - assuming the input data is already normalized",file=log,sep='\n\n',append=TRUE)
    cat("document.write('<p>Warning: No normalization will be performed - assuming the input data is already normalized</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
    normalized = raw
    HKnormalized = HKdata
    HKs = NULL
    normalizedall = list(); normalizedall[[1]] = rep(1,dim(raw)[1])
  }
  ### if a list of HKs to use is input, check data:
  if(method == "specific HKs")
  {
    # make sure a vector of HKs has been input.  If not, go ahead with the geNorm algorithm:
    if(length(selected.HKs)==0)
    {
      warning("Warning: No housekeepers were preselected - using the geNorm algorithm to select housekeepers")
      cat("LOG: Warning: No housekeepers were preselected - using the geNorm algorithm to select housekeepers",file=log,sep='\n\n',append=TRUE)
      cat("document.write('<p>Warning: No housekeepers were preselected - using the geNorm algorithm to select housekeepers</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
      method="choose HKs"; choose.method = "geNorm"
    }
    # make the vector of HKs belongs to the codeset:
    chosen.outside.codeset = unique(setdiff(selected.HKs,codeset.genes))
    if(length(chosen.outside.codeset)>0)
    {
      warning(paste("Warning: The following genes do not appear in the codeset:",paste(chosen.outside.codeset,collapse=""),". Using the remaining ",length(unique(selected.HKs))-length(unique(chosen.outside.codeset))," preselected housekeepers.",sep=""))
      cat(paste("LOG: Warning: The following genes do not appear in the codeset:",paste(chosen.outside.codeset,collapse=""),". Using the remaining ",length(unique(selected.HKs))-length(unique(chosen.outside.codeset))," preselected housekeepers.",sep=""),file=log,sep='\n\n',append=TRUE)
      cat(paste("document.write('<p>Warning: The following genes do not appear in the codeset:",paste(chosen.outside.codeset,collapse=""),". Using the remaining ",length(unique(selected.HKs))-length(unique(chosen.outside.codeset))," preselected housekeepers.</p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
      selected.HKs=intersect(selected.HKs,codeset.genes)
    }
    chosen.outside.HKcandidates = unique(setdiff(selected.HKs,codeset.HKs))
    if(length(chosen.outside.HKcandidates)>0)
    {
      warning(paste("Warning: The list of selected housekeepers includes the following genes chosen as informative in cancer: ",
                    paste(chosen.outside.HKcandidates,collapse="")))
      cat(paste("LOG: Warning: The list of selected housekeepers includes the following genes chosen as informative in cancer: ",
                paste(chosen.outside.HKcandidates,collapse="")),file=log,sep='\n\n',append=TRUE)
      cat(paste("document.write('<p>Warning: The list of selected housekeepers includes the following genes chosen as informative in cancer: ",
                paste(chosen.outside.HKcandidates,collapse=""),"</p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
    }
  }
  ### if a list of HKs to use is input, use those HKs to normalize the data:
  if(method == "specific HKs")
  {
    ## use only the selected HKs:
    normalizedall = normalize.given.HKs(raw,HKdata,selected.HKs) 
    normalized = normalizedall$normalized
    HKnormalized = normalizedall$HKnormalized
    HKs = selected.HKs
  }
  ### select normalization genes in a data-driven manner:
  if(method == "choose HKs")
  {
    # make sure there's an allowed method chosen:
    if(!is.element(choose.method,c("geNorm")))
    {
      warning("Warning: No method for choosing housekeepers selected - using the geNorm algorithm")
      cat("LOG: Warning: No method for choosing housekeepers selected - using the geNorm algorithm",file=log,sep='\n\n',append=TRUE)
      cat("document.write('<p>Warning: No method for choosing housekeepers selected - using the geNorm algorithm</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
      choose.method="geNorm"
    }
    
    if(choose.method=="geNorm")
    {
      minNrHKs = 2
      # run geNorm:
      temp = selectHKs(HKdata,method="geNorm",minNrHKs=minNrHKs,log=TRUE,Symbols = dimnames(HKdata)[[2]],trace=FALSE)
      ## select top number of genes:
      # if you decided beforehand how many to take:
      if(!auto.HKs)
      {
        HKs = as.vector(temp$ranking)[1:n.HKs]
      }
      # if you want to choose the top housekeepers dynamically:
      nselect=n.HKs
      if(auto.HKs)
      {
          nselect = dim(HKdata)[2]-(1:length(temp$variation))[temp$variation==min(temp$variation)]+1
          # require at least 6 HKs:
          nselect = max(nselect,6)
          HKs = as.vector(temp$ranking)[1:nselect] 
      }
      
      
     # plot the results as a sanity check:
     for(r in 1:length(plottypearg)){
       plottype=plottypearg[r];
       tempfilename = drawplot(filename=paste(path.to.normalization.results,"//HK selection details - pairwise variance",sep=""),plottype)
	 tempfilename=gsub(path.results,"results",tempfilename)
    
     plot(temp$variation,
          col=c(rep("grey",dim(HKdata)[2]-nselect),rep("black",nselect)),
          pch=16,
          main="Genes selected using geNorm",
          ylab="Pairwise variation during stepwise selection",xlab="Order removed")
     abline(v=dim(HKdata)[2]-nselect+.5)
     legend("topleft",col=c("black","grey"),pch=c(16,1),legend=c("selected","unselected"))
     dev.off()}
    
    }
   
   # now normalize:
   normalizedall = normalize.given.HKs(raw,HKdata,HKs) 
   normalized = normalizedall$normalized
   HKnormalized = normalizedall$HKnormalized
      
   # plot sample means before and after normalization to show how successful it was:
   for(r in 1:length(plottypearg)){
     plottype=plottypearg[r];
     tempfilename = drawplot(filename=paste(path.to.normalization.results,"//total expression before and after normalization",sep=""),plottype,height=1.5)
	 tempfilename=gsub(path.results,"results",tempfilename)
   par(mfrow=c(2,1))
   hist(apply(raw,1,mean),xlim=range(apply(raw,1,mean)),main="Raw data",xlab="Samples' mean log expression of all genes",breaks=20)
   hist(apply(cbind(normalized,HKnormalized),1,mean),xlim=range(apply(raw,1,mean)),main="Normalized data",xlab="Samples' mean log expression of all genes",breaks=20)
   dev.off()}
  }
print("Normalized data")
cat("LOG:Normalized data",file=log,sep='\n\n',append=TRUE)
cat("document.write('<p>Normalized data</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
cat("document.write('		  	<ul>');\n",file=paste(path.inc,"//panel2_5.js",sep=""),append=TRUE)

  
  # write csv of selected HKs:
  if(method!="none")
  {
    HKstowrite = as.data.frame(HKs)
    names(HKstowrite) = "HKs in the order by which they were selected"
    write.csv(HKstowrite,file=paste(path.to.normalization.results,"//selected housekeepers.csv",sep=""),row.names=FALSE)

	file=paste("results//Normalization","//selected housekeepers.csv",sep="")
	strTemp=paste("document.write('	       <li><a href=\"",file,"\">View Selected HK Genes</a></li>')\n",sep="")
	cat(strTemp,file=paste(path.inc,"//panel2_5.js",sep=""),append=TRUE)

  }
  # threshold normalized data at 1:
  normalized = replace(normalized,normalized<0,0)
  HKnormalized = replace(HKnormalized,HKnormalized<0,0)
  out = list(normalized=normalized,HKnormalized=HKnormalized,HKs=HKs,norm.factors=normalizedall$norm.factors)
  # write csv of normalized data:
  write.csv(normalized,file=paste(path.to.normalization.results,"//normalized log2 data.csv",sep=""))

file=paste("results//Normalization","//normalized log2 data.csv",sep="")
strTemp=paste("document.write('	       <li><a href=\"",file,"\">View Normalized Data</a></li>')\n",sep="")
cat(strTemp,file=paste(path.inc,"//panel2_5.js",sep=""),append=TRUE)
cat("document.write('		  	<ul>');\n",file=paste(path.inc,"//panel2_5.js",sep=""),append=TRUE)
cat("document.write('		</div>');\n",file=paste(path.inc,"//panel2_5.js",sep=""),append=TRUE)

print("Created normalized data files")
cat("LOG:Created normalized data files",file=log,sep='\n\n',append=TRUE)
cat("document.write('<p>Created normalized data files</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
return(out)
}


###########################################################################
###########################################################################
# data exploration and QC
###########################################################################
###########################################################################


runQC = function(normalized,HKnormalized,HKs,covariates,covariates.type,gene.sets=NULL,gene.set.names=NULL,annotcols,annotcols2,pruneflag=FALSE)  
{

print("Starting QC")
cat("LOG:Starting QC",file=log,sep='\n\n',append=TRUE)
cat("document.write('<p>Starting QC</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
#---HTML report code injection---#

#write QC include file
#static content - left menu
cat("document.write('	    <div class=\"large-2 columns\" id=\"side_menu\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('    		  <H4>Pathways</H4>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('              <ul class=\"button-group tabs vertical\" data-tab>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title active\"><a href=\"#summary-QC\"><b>Summary</b></a></li>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#apop-QC\">Apoptosis</a></li>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#cc-QC\">Cell Cycle</a></li>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#chrommod-QC\">Chromatin Modification</a></li>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('			     <li class=\"tab-title\"><a href=\"#dnarepair-QC\">DNA Repair</a></li>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#hh-QC\">Hedgehog</a></li>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#stat-QC\">JAK-STAT</a></li>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#mapk-QC\">MAPK</a></li>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#notch-QC\">Notch</a></li>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#pi3k-QC\">PI3K</a></li>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#ras-QC\">RAS</a></li>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#tgfb-QC\">TGF-Beta</a></li>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#txmisreg-QC\">TX Mis Regulation</a></li>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#wnt-QC\">Wnt</a></li>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('              </ul>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('          </div>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('		 <div class=\"large-10 columns\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('		   <div class=\"tabs-content\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)

cat("document.write('                   <div class=\"content active\" id=\"summary-QC\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('	 			   <nav class=\"top-bar\" data-topbar>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('	 			         <ul class=\"button-group  tabs\" data-tab>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('						   <li class=\"tab-title active\"><a href=\"#summary-QC-Summary\">Summary</a></li>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)

#### QC CODE BLOCK
##QCMENU
#loop through covariates to create dynamic menu items
allcovariatesT = as.data.frame(covariates)
if(dim(allcovariatesT)[2]>1)
{
  cat("document.write('						   <li class=\"tab-title\"><a href=\"#summary-QC-PairPlots\">Pair Plots</a></li>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
}
  if(dim(allcovariatesT)[2]>0)
  {

cat("document.write('						   <li class=\"tab-title\"><a href=\"#summary-QC\" data-dropdown=\"QCPCA\">PCA Plots</a></li>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('	 			              <ul id=\"QCPCA\" class=\"f-dropdown\" data-dropdown-content data-tab>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)


    for(i in 1:dim(allcovariatesT)[2])
    {
		catTemp <-dimnames(allcovariatesT)[[2]][i]
		catTemp = gsub("\\.","",catTemp)
		catTemp = gsub("\\s","",catTemp)
		lineTemp = paste("document.write('						   <li><a href=\"#summary-QC-",catTemp,"\">",dimnames(allcovariatesT)[[2]][i],"</a></li>');\n",sep="")
		cat(lineTemp,file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
    }
	
     cat("document.write('	 			              </ul>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)

  }

cat("document.write('	 			         </ul>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('	 			   </nav>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('		          <div class=\"tabs-content\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)

cat("document.write('              	    <div class=\"content active\" id=\"summary-QC-Summary\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('              	    		  	<ul class=\"small-block-grid-2\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('              	    				<li>Correlation between all samples<br><img src=\"results/QC/correlation between samples - heatmap.png\" ></img></li>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('              	    				<li>Heatmap of all genes x samples<br><img src=\"results/QC/heatmap of all genes X samples.png\" ></img></li>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('              	    				<li>Mean vs var for all genes<br><img src=\"results/QC/mean vs var for all genes.png\" ></img></li>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('              	    				<li>P-value Histograms<br><img src=\"results/QC/p-value histograms.png\"></img></li>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)

if(pruneflag)
{
file=paste("./genes removed for low signal.csv",sep="")
strTemp=paste("document.write('              	    				<li><ul><li><a href=\"",file,"\">View Genes Removed Due (Low Signal)</a></li></ul></li>')\n",sep="")
cat(strTemp,file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
}


cat("document.write('              	    			</ul>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)	
cat("document.write('                    </div>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)

### -- end HTML


  alldata = cbind(normalized,HKnormalized)
  genecols = c(rep(codecols[1],dim(normalized)[2]),rep(codecols[2],dim(HKnormalized)[2]))
  genecols[is.element(dimnames(alldata)[[2]],HKs)]=codecols[3]
  
  # plot mean vs. sd in normalized data, color points by type: endog, HK (selected), HK (unselected)
  means = apply(alldata,2,mean)
  vars = apply(alldata,2,var)
  for(r in 1:length(plottypearg)){
    plottype=plottypearg[r];
    tempfilename = drawplot(filename=paste(path.to.QC.results,"//mean vs var for all genes",sep=""),plottype)
    tempfilename=gsub(path.results,"results",tempfilename)
  par(xpd=TRUE)
  par(mar=c(5,4,4,1))
  plot(vars~means,col=genecols,pch=(1+15*is.element(dimnames(alldata)[[2]],dimnames(HKnormalized)[[2]])),xlab="Mean log expression",ylab="Variance")
  legend("topleft",inset=c(0,-0.16),col=codecols[c(1,3,2)],pch=c(1,16,16),legend=c("Cancer pathway gene","Housekeepers used in normalization","Housekeepers - unused"))
  topvars = order(vars,decreasing=TRUE)[1:20]
  text(means[topvars],vars[topvars],dimnames(alldata)[[2]][topvars])
  dev.off()}

  
  #### plot relationships btw covariates:

  if(dim(covariates)[2]>1)
  {
	# BEGIN HTML
	cat("document.write('              	    <div class=\"content\" id=\"summary-QC-PairPlots\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
	cat("document.write('              	    		  	<ul class=\"small-block-grid-2\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
	# END HTML
	print("Creating QC plots")
	cat("LOG:Creating QC plots",file=log,sep='\n\n',append=TRUE)
	cat("document.write('<p>Creating QC plots</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
    # plot a matrix of results.  Scatterplots/boxplots/tables on top right, p-values for association on bottom left:
    for(i in 1:(dim(covariates)[2]-1))
    {
      for(j in (i+1):dim(covariates)[2])
      {
        for(r in 1:length(plottypearg)){
          plottype=plottypearg[r];
          #if(!(covariates.type[i]=="categorical")&(covariates.type[j]=="categorical"))
          #{
              tempfilename = drawplot(filename=paste(path.to.QC.results,"//pairs plot of covariates - ",dimnames(covariates)[[2]][i]," vs ",dimnames(covariates)[[2]][j],sep=""),plottype)
			tempfilename=gsub(path.results,"results",tempfilename)
		   # BEGIN HTML - need to loop thru pair plots
		   if(r==1)
		   {
			lineTemp = paste("document.write('              	    		  	   <li>",dimnames(covariates)[[2]][i]," vs ",dimnames(covariates)[[2]][j],"<br><img src=\"",tempfilename,".png\"></img></li>');\n",sep="")
			cat(lineTemp,file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)

		   }
		   #END HTML

          #}
        # scatterplot between continuous covariates:
        if((covariates.type[i]=="continuous")&(covariates.type[j]=="continuous"))
        {
          plot(covariates[,i],covariates[,j],xlab=dimnames(covariates)[[2]][i],ylab=dimnames(covariates)[[2]][j])
        }
        # boxplot between cont and cat:
        par(mar=c(5.1,6,4.1,2.1))
        if((covariates.type[i]=="continuous")&(covariates.type[j]=="categorical"))
        {
          par(mar=c(8,6,2,1))
          boxplot(covariates[,i]~covariates[,j],ylab=dimnames(covariates)[[2]][i],las=2,border=annotcols2[[j]],
                  main=paste(dimnames(covariates)[[2]][i],"vs.",dimnames(covariates)[[2]][j])) 
          
        }
        if((covariates.type[j]=="continuous")&(covariates.type[i]=="categorical"))
        {
          par(mar=c(8,6,2,1))
          boxplot(covariates[,j]~covariates[,i],ylab=dimnames(covariates)[[2]][j],las=2,border=annotcols2[[i]],
                  main=paste(dimnames(covariates)[[2]][j],"vs.",dimnames(covariates)[[2]][i]))  
        }
        # compare cat to cat:
        if((covariates.type[i]=="categorical")&(covariates.type[j]=="categorical"))
        {
          tab = as.matrix(table(covariates[,c(i,j)]))
          flip = dim(tab)[1]>dim(tab)[2]
          annotcodecol = i
          if(flip)
          {
            tab = t(tab)
            # color by i or j?
            annotcodecol = j
          }
          barplot(tab,col=annotcols2[[annotcodecol]],las=2,main=paste(dimnames(covariates)[[2]][j],"vs.",dimnames(covariates)[[2]][i]))  
          
          #write.csv(tab,file=paste(path.to.QC.results,"//pairs plot of covariates - ",dimnames(covariates)[[2]][i]," vs ",dimnames(covariates)[[2]][j],".csv",sep=""))
        }
        #if(!(covariates.type[i]=="categorical")&(covariates.type[j]=="categorical")){dev.off()}}          
			  dev.off()}
      }
    }
	# BEGIN HTML - closing out div for dynamic pair plots
	cat("document.write('              	    			</ul>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)	
	cat("document.write('                    </div>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
	# END HTML
  }
  write.csv(covariates,file="QC module - sample annot.csv")

  ### heatmap of all data, arranged by pathway membership  
  #hm1 = heatmap.2(t(normalized),scale="row")
  # first, fill in the row colors matrix:
  if(length(covariates)>0)
  {
    rowcols = annotcols
    cat("dimnames(covariates):",file=log,sep='\n',append=TRUE)
    suppressWarnings(write.table(dimnames(covariates)[[2]],file=log,sep='\t',append=TRUE))
    suppressWarnings(write.table(dimnames(covariates)[[1]][1:5],file=log,sep='\t',append=TRUE))
    cat("dim(rowcols)",file=log,sep='\n',append=TRUE)
    cat(dim(rowcols),file=log,sep='\n',append=TRUE)
    dimnames(rowcols)=dimnames(covariates)
    if(i==1){rowcols = cbind(rowcols,"white")}
  }
#  if(length(gene.sets)>0)
#  {
#    bigmatrix = t(normalized)
#    pathsizes = dim(bigmatrix)[1]
#  }
  if(length(gene.sets)>0)
  {
#    bigmatrix = c()
#    pathsizes = c()
#    pathbreaks = c()
#    pathcolors = c()
    gene.sets = gene.sets[match(dimnames(normalized)[[2]],dimnames(gene.sets)[[1]]),]
    for(k in 1:dim(gene.sets)[2])
    {
      print(paste("Creating QC heatmaps for pathway ",k,"/",dim(gene.sets)[2]))
      cat(paste("LOG: Creating QC heatmaps for pathway ",k,"/",dim(gene.sets)[2]),file=log,sep='\n\n',append=TRUE)
      cat(paste("document.write('<p>Creating QC heatmaps for pathway ",k,"/",dim(gene.sets)[2],"</p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
      
      # isolate data matrix from the current gene set:
      pathmatrix = t(normalized[,dimnames(gene.sets)[[1]][gene.sets[,k]==1]])
      # run a heatmap on the pathway matrix to get a better ordering of the genes
      for(r in 1:length(plottypearg)){
        plottype=plottypearg[r];
        tempfilename = drawplot(filename=paste(path.to.QC.results,"//heatmap of data - ",dimnames(gene.sets)[[2]][k],sep=""),plottype,
               width=(1.5+1*(dim(normalized)[1]>50)),height=1.75,heatmapres=TRUE)
              #width=(3+2*(dim(normalized)[1]>50)),height=3.5)
	   tempfilename=gsub(path.results,"results",tempfilename)
      par(mar=c(10,4,2,1))
      if(length(covariates)>0){ heatmap.plus(pathmatrix,symm=FALSE,Colv=TRUE,Rowv = TRUE,ColSideColors=rowcols,margins = c(8,8)) }
      if(length(covariates)==0){ heatmap.plus(pathmatrix,symm=FALSE,Colv=TRUE,Rowv = TRUE) }
      dev.off()}
      
      ## add to the main reordered data matrix:
      #hmt = heatmap.2(pathmatrix,scale="row",trace="none")  #<-  moved this down from 5 or 6 rows up; shouldn't change anything save for solving the multiple heatmaps per pdf issue
      #pathmatrix = pathmatrix[hmt$rowInd,]
      #bigmatrix = rbind(bigmatrix,pathmatrix)
      #pathsizes[k] = dim(normalized[,gene.sets[,k]==1])[2]   
      #pathbreaks[k] = sum(pathsizes)+1
      # color pathways:
      #if(k%%2==0){pathcolors = c(pathcolors,rep("grey",pathsizes[k]))}
      #if(k%%2==1){pathcolors = c(pathcolors,rep("black",pathsizes[k]))}
    }
  }
  #pathcolors = cbind("white",pathcolors)

  # plot the final heatmap:
#dimnames(pathcolors)[[2]]=c("","")
  for(r in 1:length(plottypearg)){
  plottype=plottypearg[r];
  tempfilename = drawplot(filename=paste(path.to.QC.results,"//heatmap of all genes X samples",sep=""),plottype,height=2,width=1.5,heatmapres=TRUE)
  #tempfilename = drawplot(filename=paste(path.to.QC.results,"//heatmap of all genes X samples",sep=""),plottype,height=4,width=3)
tempfilename=gsub(path.results,"results",tempfilename)
  par(mar=c(10,8,2,1))
  if(length(covariates)>0){ heatmap.plus(t(normalized),symm=FALSE,Colv=TRUE,Rowv = TRUE,labRow=NA,margins = c(8,8),ColSideColors=(rowcols)) }
  if(length(covariates)==0){ heatmap.plus(t(normalized),symm=FALSE,Colv=TRUE,Rowv = TRUE,labRow=NA,margins = c(8,8)) }
  dev.off()}
  
  
  #### correlation matrix of all samples:  
  cormat = cor(t(normalized))
  neg.elements = sign(cormat)==-1
  cormat.rescaled = cormat/abs(min(cormat))
  cormat2 = replace(cormat,neg.elements,cormat.rescaled[neg.elements])
write.csv(cormat2,file = paste(path.to.csvs,"//QC - correlation matrix of samples.csv",sep=""))
  for(r in 1:length(plottypearg)){
    plottype=plottypearg[r];
    tempfilename = drawplot(filename=paste(path.to.QC.results,"//correlation between samples - heatmap",sep=""),plottype,width=2,height=2,heatmapres=TRUE)
    #tempfilename = drawplot(filename=paste(path.to.QC.results,"//correlation between samples - heatmap",sep=""),plottype,width=4,height=4)
tempfilename=gsub(path.results,"results",tempfilename)
    par(mar=c(10,4,2,1))
    hmcols<-colorRampPalette(c("blue","antiquewhite3",codecols2[4]))(256)
    hmcols = hmcols
    if(length(covariates)>0){ heatmap.plus(cormat2,col=hmcols,symm=TRUE,margins=c(7,7),ColSideColors=(rowcols),RowSideColors=(rowcols)) }
    if(length(covariates)==0){ heatmap.plus(cormat2,col=hmcols,symm=TRUE,margins=c(7,7)) }
    dev.off()}


  ### PCA-based QC:
  pc = prcomp(alldata)
  ## automatically identify outliers in PCA:
  x = pc$x[,1:4]

  # outliers: anything > 3sds from mean:
  outliers = matrix(FALSE,dim(x)[1],dim(x)[2])
  dimnames(outliers)[[1]]=dimnames(alldata)[[1]]
  dimnames(outliers)[[2]] = paste("Outlier on PC",1:dim(x)[[2]])
  for(i in 1:dim(x)[2])
  {
    outliers[abs(x[,i]-mean(x[,i]))>3*sd(x[,i]),i]=TRUE   
  }
  # output list of outliers along with PC on which they're an outlier:
  if(sum(rowSums(outliers)>0)>1){outliers = outliers[rowSums(outliers)>0,]}
  if(sum(rowSums(outliers)>0)==1)
  {
    outliername = dimnames(outliers)[[1]][rowSums(outliers)>0] 
    outliers = t(as.matrix(outliers[rowSums(outliers)>0,]))
    dimnames(outliers)[[1]]=outliername
  }
  if(sum(rowSums(outliers)>0)>0){write.csv(outliers,file=paste(path.to.QC.results,"//outliers in first 4 principal components.csv",sep=""))}

## PC pairs plots colored by covariates:
  allcovariates = as.data.frame(covariates)
  if(dim(allcovariates)[2]>0)
  {
    for(i in 1:dim(allcovariates)[2])
    {
      if(covariates.type[i]=="continuous")
      {
        pointcols = colorRampPalette(c("deepskyblue2","grey","firebrick2"))( dim(allcovariates)[1] )[rank(allcovariates[,i])]
        for(r in 1:length(plottypearg)){
          plottype=plottypearg[r];
          tempfilename = drawplot(filename=paste(path.to.QC.results,"//principal components colored by",dimnames(allcovariates)[[2]][i],sep=""),plottype)
		tempfilename=gsub(path.results,"results",tempfilename)
		# BEGIN HTML 
		if(r==1)
		{
			catTemp <-dimnames(allcovariates)[[2]][i]
			catTemp = gsub("\\.","",catTemp)
       		catTemp = gsub("\\s","",catTemp)
			strTemp=paste("document.write('              	    <div class=\"content\" id=\"summary-QC-",catTemp,"\">');\n",sep="")
			cat(strTemp,file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
			cat("document.write('        		  	   <ul class=\"small-block-grid-1\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)

			strTemp=paste("document.write('                        <li>Principle Component by ",dimnames(allcovariates)[[2]][i],"<br><img src=\"",tempfilename,".png\"></img><img src=\"",tempfilename," legend.png\"></img></li>');\n",sep="")
			cat(strTemp,file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
			
			cat("document.write('                       </ul>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
			cat("document.write('                    </div>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
		}
		# END HTML 

        pairs(pc$x[,1:4],col=pointcols,xaxt="n",yaxt="n",
              labels = paste("PC",1:4," - ",round((pc$sdev^2)[1:4]/sum(pc$sdev^2),2),sep=""))
        dev.off()}
        for(r in 1:length(plottypearg)){
          plottype=plottypearg[r];
          tempfilename = drawplot(filename=paste(path.to.QC.results,"//principal components colored by",dimnames(allcovariates)[[2]][i]," legend",sep=""),plottype,width=.5)
		tempfilename=gsub(path.results,"results",tempfilename)
        frame()
        legend("center",pch=1,col=c(NA,"deepskyblue2","firebrick2"),legend=c(main=names(allcovariates)[i],"low","high"))
        dev.off()}
      }
      if(covariates.type[i]=="categorical")
      {
        allcovariates[,i] = as.factor(allcovariates[,i])
        levels = unique(allcovariates[,i])
        pointcols = annotcols[,i]             
        for(r in 1:length(plottypearg)){
          plottype=plottypearg[r];
          tempfilename = drawplot(filename=paste(path.to.QC.results,"//principal components colored by",dimnames(allcovariates)[[2]][i],sep=""),plottype)
		tempfilename=gsub(path.results,"results",tempfilename)
		# BEGIN HTML 
		if(r==1)
		{
			
			catTemp <-dimnames(allcovariates)[[2]][i]
			catTemp = gsub("\\.","",catTemp)
			catTemp = gsub("\\s","",catTemp)
			strTemp=paste("document.write('              	    <div class=\"content\" id=\"summary-QC-",catTemp,"\">');\n",sep="")
			cat(strTemp,file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
			cat("document.write('        		  	   <ul class=\"small-block-grid-1\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)

			strTemp=paste("document.write('                        <li>Principle Component by ",dimnames(allcovariates)[[2]][i],"<br><img src=\"",tempfilename,".png\"></img><img src=\"",tempfilename," legend.png\"></img></li>');\n",sep="")
			cat(strTemp,file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
			
			cat("document.write('                       </ul>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
			cat("document.write('                    </div>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
		}
		# END HTML 
        pairs(pc$x[,1:4],col=pointcols,xaxt="n",yaxt="n",
              labels = paste("PC",1:4," - ",round((pc$sdev^2)[1:4]/sum(pc$sdev^2),2),sep=""))
        dev.off()}
        for(r in 1:length(plottypearg)){
          plottype=plottypearg[r];
          tempfilename = drawplot(filename=paste(path.to.QC.results,"//principal components colored by",dimnames(allcovariates)[[2]][i]," legend",sep=""),plottype,width=.5)
		tempfilename=gsub(path.results,"results",tempfilename)
        frame()
		    legend("center",pch=1,col=c(NA,annotcols2[[i]]),legend=c(names(allcovariates)[i],levels(allcovariates[,i]))) 
        dev.off()}
      }
    }
    write.csv(allcovariates,file=paste(path.to.csvs,"//QC - covariates used in p-value histograms.csv",sep=""))
    
    ### p-value histograms of univariate analyses of ALL variables:
    for(r in 1:length(plottypearg)){
      print(paste("Creating QC histograms ",r,"/",length(plottypearg)))
      cat(paste("LOG: Creating QC histograms ",r,"/",length(plottypearg)),file=log,sep='\n\n',append=TRUE)
      cat(paste("document.write('<p>Creating QC histograms ",r,"/",length(plottypearg),"</p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
    plottype=plottypearg[r];
    tempfilename = drawplot(filename=paste(path.to.QC.results,"//p-value histograms",sep=""),plottype,height=1.5,width=1.5)
	tempfilename=gsub(path.results,"results",tempfilename)    
	if(dim(allcovariates)[[2]]<=4)
    {
      par(mfrow=c(ceiling(dim(allcovariates)[[2]]/2),min(dim(allcovariates)[[2]],2)))
    }
    if(dim(allcovariates)[[2]]>4)
    {
      par(mfrow=c(ceiling(dim(allcovariates)[[2]]/3),min(dim(allcovariates)[[2]],3)))
    }
    for(i in 1:dim(allcovariates)[2])
    {
      pvals = c()
      for(j in 1:dim(normalized)[2])
      {
        mod = lm(cbind(normalized[,j],as.data.frame(allcovariates[,i])))      
        mod2 = lm(normalized[,j]~1) 
        # run LRT:
        LLR = logLik(mod)-logLik(mod2)
        df = length(mod$coef)-length(mod2$coef)
        pvals[j]=1-pchisq(2*LLR,df)
      }
      hist(pvals,main=dimnames(allcovariates)[[2]][i],breaks=(0:20)/20,xlim=c(0,1),cex.main=1.45,cex.lab=1.45,xlab="p-values")
    }
    dev.off()}
  }
print("Creating QC HTML infrastructure")
cat("LOG: Creating QC HTML infrastructure",file=log,sep='\n\n',append=TRUE)
cat("document.write('<p>Creating QC HTML infrastructure</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)

#BEGIN HTML
cat("document.write('                  </div>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('              </div>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)


#//start new pathway - apoptosis
cat("document.write('              <div class=\"content\" id=\"apop-QC\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('        		  	   <ul class=\"small-block-grid-1\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                        <li>Heatmap - Apoptosis<br><img src=\"results/QC/heatmap of data - Apop.png\"></img></li>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                       </ul>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('               </div>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
#//end new pathway

#//start new pathway - cell cycle (cc)
cat("document.write('              <div class=\"content\" id=\"cc-QC\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('        		  	   <ul class=\"small-block-grid-1\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                        <li>Heatmap - Cell Cycle<br><img src=\"results/QC/heatmap of data - CC.png\"></img></li>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                       </ul>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('               </div>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
#//end new pathway

#//start new pathway - chrommod
cat("document.write('              <div class=\"content\" id=\"chrommod-QC\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('        		  	   <ul class=\"small-block-grid-1\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                        <li>Heatmap - Chromatin Modification<br><img src=\"results/QC/heatmap of data - ChromMod.png\"></img></li>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                       </ul>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('               </div>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
#//end new pathway

#//start new pathway - DNA Repair
cat("document.write('              <div class=\"content\" id=\"dnarepair-QC\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('        		  	   <ul class=\"small-block-grid-1\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                        <li>Heatmap - DNA Repair<br><img src=\"results/QC/heatmap of data - DNARepair.png\"></img></li>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                       </ul>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('               </div>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
#//end new pathway

#//start new pathway - HedgeHog
cat("document.write('              <div class=\"content\" id=\"hh-QC\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('        		  	   <ul class=\"small-block-grid-1\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                        <li>Heatmap - HedgeHog<br><img src=\"results/QC/heatmap of data - HH.png\"></img></li>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                       </ul>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('               </div>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
#//end new pathway

#//start new pathway - Jak-Stat
cat("document.write('              <div class=\"content\" id=\"stat-QC\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('        		  	   <ul class=\"small-block-grid-1\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                        <li>Heatmap - JAK-STAT<br><img src=\"results/QC/heatmap of data - STAT.png\"></img></li>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                       </ul>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('               </div>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
#//end new pathway

#//start new pathway - MAPK
cat("document.write('              <div class=\"content\" id=\"mapk-QC\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('        		  	   <ul class=\"small-block-grid-1\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                        <li>Heatmap - MAPK<br><img src=\"results/QC/heatmap of data - MAPK.png\"></img></li>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                       </ul>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('               </div>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
#//end new pathway

#//start new pathway - Notch
cat("document.write('              <div class=\"content\" id=\"notch-QC\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('        		  	   <ul class=\"small-block-grid-1\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                        <li>Heatmap - Notch<br><img src=\"results/QC/heatmap of data - NOTCH.png\"></img></li>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                       </ul>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('               </div>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
#//end new pathway

#//start new pathway - PI3K
cat("document.write('              <div class=\"content\" id=\"pi3k-QC\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('        		  	   <ul class=\"small-block-grid-1\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                        <li>Heatmap - PI3K<br><img src=\"results/QC/heatmap of data - PI3K.png\"></img></li>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                       </ul>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('               </div>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
#//end new pathway

#//start new pathway - RAS
cat("document.write('              <div class=\"content\" id=\"ras-QC\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('        		  	   <ul class=\"small-block-grid-1\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                        <li>Heatmap - RAS<br><img src=\"results/QC/heatmap of data - RAS.png\"></img></li>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                       </ul>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('               </div>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
#//end new pathway

#//start new pathway - TGFB
cat("document.write('              <div class=\"content\" id=\"tgfb-QC\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('        		  	   <ul class=\"small-block-grid-1\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                        <li>Heatmap - TGF-Beta<br><img src=\"results/QC/heatmap of data - TGFB.png\"></img></li>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                       </ul>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('               </div>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
#//end new pathway

#//start new pathway - TxMisReg
cat("document.write('              <div class=\"content\" id=\"txmisreg-QC\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('        		  	   <ul class=\"small-block-grid-1\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                        <li>Heatmap - TX MisRegulation<br><img src=\"results/QC/heatmap of data - TXmisReg.png\"></img></li>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                       </ul>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('               </div>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
#//end new pathway

#//start new pathway - Wnt
cat("document.write('              <div class=\"content\" id=\"wnt-QC\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('        		  	   <ul class=\"small-block-grid-1\">');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                        <li>Heatmap - Wnt<br><img src=\"results/QC/heatmap of data - Wnt.png\"></img></li>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('                       </ul>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('               </div>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
#//end new pathway


#//end of file
cat("document.write('		    </div>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)
cat("document.write('		  </div>');\n",file=paste(path.inc,"//panel2_6.js",sep=""),append=TRUE)

# END HTML
print("Finished QC")
cat("LOG:Finished QC",file=log,sep='\n\n',append=TRUE)
cat("document.write('<p>Finished QC</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
} 


#################################
#################################
# Pathway scoring code using PCA, Pathifier or pre-defined scores
#################################
#################################


build.pathway.scores = function(normalized,method=c("PC1","Pathifier","Pre-defined"),
                                gene.sets,gene.set.names,is.normal=NULL,path.to.predefined.scores=NULL,plot.cormatrix=TRUE,draw.networks=TRUE,
                                pairs.plot.of.scores=TRUE,PC.pairs.plot.of.scores=TRUE,plot.against.covariates=TRUE,
                                covariates=NULL,covariate.type=NULL,adjust.for=NULL,genes.to.omit=NULL,annotcols,annotcols2)
{
print("Starting pathway scoring")
cat("LOG:Starting pathway scoring",file=log,sep='\n\n',append=TRUE)
cat("document.write('<p>Starting pathway scoring</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)


###pathscore html
 minsd = 0.05 # the sd due to noise, plus a buffer to minimize the impact of low var genes: every gene with a sd lower than this will be rescaled by minsd instead
 # create a dataset of residuals on which to fit scores:
 if(method != "Pre-defined")
 {
  adjusted = normalized[,!genes.to.omit]
 #adjusted = normalized
  gene.sets = gene.sets[!genes.to.omit,]
 }
 if(method == "Pre-defined")
 {
 adjusted = normalized
 }
 if(length(adjust.for)>0)
 {
 # Check the degrees of freedom:
 temp = cbind(normalized[,1],adjust.for)
 if(summary(lm(temp))$df[2]<1)
 {
 warning("Warning: Too many parameters or levels to adjust for given the number of observations - Pathway scoring will not be run.") 
 cat("LOG: Too many parameters or levels to adjust for given the number of observations - Pathway scoring will not be run.",file=log,sep='\n\n',append=TRUE)
 cat("document.write('<p>Too many parameters or levels to adjust for given the number of observations - Pathway scoring will not be run.</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
 return(scores=NULL)
 }
 adjusted=adjusted*NA
 for(i in 1:dim(adjusted)[2])
 {
 # combine with adjust.for to create a data frame on which to run a lm:
 temp = cbind(normalized[,i],adjust.for)
 mod = lm(temp)
 # fill in the "adjusted" data frame with the residuals of the lm:
 adjusted[names(mod$resid),i] = mod$resid
 }
 # remove NA rows:
 adjusted = adjusted[rowSums(is.na(adjusted))==0,]
 }



#---HTML report code injection---#

#write pathwayscore include file
#static content - left menu
cat("document.write('	    <div class=\"large-2 columns\" id=\"side_menu\">');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
cat("document.write('    		  <H4>Pathways</H4>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
cat("document.write('              <ul class=\"button-group tabs vertical\" data-tab>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title active\"><a href=\"#summary-pathscore\"><b>Summary</b></a></li>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#Apop-pathscore\">Apoptosis</a></li>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#CC-pathscore\">Cell Cycle</a></li>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#ChromMod-pathscore\">Chromatin Modification</a></li>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#DNARepair-pathscore\">DNA Repair</a></li>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#HH-pathscore\">Hedgehog</a></li>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#STAT-pathscore\">JAK-STAT</a></li>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#MAPK-pathscore\">MAPK</a></li>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#Notch-pathscore\">Notch</a></li>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#PI3K-pathscore\">PI3K</a></li>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#RAS-pathscore\">RAS</a></li>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#TGFB-pathscore\">TGF-Beta</a></li>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#TXmisReg-pathscore\">TX Mis Regulation</a></li>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#Wnt-pathscore\">Wnt</a></li>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
cat("document.write('              </ul>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
cat("document.write('          </div>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
cat("document.write('		 <div class=\"large-10 columns\">');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
cat("document.write('		   <div class=\"tabs-content\">');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
cat("document.write('                   <div class=\"content active\" id=\"summary-pathscore\">');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
if(plot.against.covariates&(dim(covariates)[2]>0))
{
cat("document.write('	 			   <nav class=\"top-bar\" data-topbar>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
cat("document.write('	 			         <ul class=\"button-group  tabs\" data-tab>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
cat("document.write('						   <li class=\"tab-title active\"><a href=\"#summary-pathscore-Summary\">Summary</a></li>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
cat("document.write('						   <li class=\"tab-title\"><a href=\"#summary-pathscore\" data-dropdown=\"PSPCA\">PCA Plots</a></li>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
cat("document.write('	 			         </ul>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
cat("document.write('	 			   </nav>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
}
cat("document.write('		          <div class=\"tabs-content\">');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
cat("document.write('              	    <div class=\"content active\" id=\"summary-pathscore-Summary\">');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
cat("document.write('              	    		  	<ul class=\"small-block-grid-2\">');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
cat("document.write('              	    				<li>Pathway Score Correlation<br><img src=\"results/Pathway scoring/pathway scores correlation heatmap.png\" ></img></li>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
cat("document.write('                    				<li>Pathway Scores Heatmap<br><img src=\"results/Pathway scoring/pathway scores heatmap.png\"></img></li>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)




#END HTML 



write.csv(adjusted,file=paste(path.to.csvs,"//pathway scores - residuals of data after adjusting for selected covariates.csv",sep=""))
#
  # if a covariate giving normal reference levels is present, create the is.normal variable
  if(length(is.normal)>0)
  {
    is.normal2 = rep(NA,length(is.normal))
    is.normal2[is.normal == levels(is.normal)[1]]=TRUE
    is.normal2[is.normal != levels(is.normal)[1]]=FALSE
    is.normal=is.normal2
  }
  cat("table(is.normal):",file=log,sep='\n',append=TRUE)
  cat(table(is.normal),file=log,sep='\n',append=TRUE)

  # check that there are normals so that pathifier can run:
  if(method=="Pathifier") 
  {
    if(length(is.normal)==0)
    {
      warning("Warning: No samples were marked as normal references - Pathifier will not be run")
      cat("LOG: Warning: No samples were marked as normal references - Pathifier will not be run",file=log,sep='\n\n',append=TRUE)
      cat("document.write('<p>Warning: No samples were marked as normal references - Pathifier will not be run</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
      method = "none"
    }
  }
  
  # fit pathifier scores:
  if(method=="Pathifier")
  {
    print("Using Pathifier")
    cat("LOG:Using Pathifier",file=log,sep='\n\n',append=TRUE)
    cat("document.write('<p>Using Pathifier</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
    K = length(gene.set.names)
    scores = matrix(NA,dim(adjusted)[1],K)
    dimnames(scores)[[1]]=dimnames(adjusted)[[1]]
    dimnames(scores)[[2]]=gene.set.names
    listofpathwaygenes = list()
    for(k in 1:K)
    {
      listofpathwaygenes[[k]] = dimnames(gene.sets)[[1]][gene.sets[,gene.set.names[k]]==1]  
      names(listofpathwaygenes)[k]=gene.set.names[k]
    }

    cat("dim(pathifier data):",file=log,sep='\n',append=TRUE)
    cat(dim(t(adjusted)),file=log,sep='\n',append=TRUE)
    
    # arguments for Pathifier:
    min_exp = log2(5); if(length(adjust.for)>0){min_exp=-Inf}
    # run Pathifier:
    print("Running Pathifier")
    cat("LOG:Running Pathifier",file=log,sep='\n\n',append=TRUE)
    cat(paste("document.write('<p>Running Pathifier ...</p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
    pathif = quantify_pathways_deregulation(data=t(adjusted),allgenes=dimnames(adjusted)[[2]],syms=listofpathwaygenes,   
                                            pathwaynames=gene.set.names,normals=is.normal,
                                            min_exp=min_exp,min_std=minsd)
    print("Finished Pathifier")
    cat("LOG:Finished Pathifier",file=log,sep='\n\n',append=TRUE)
    cat(paste("document.write('<p>Finished Pathifier</p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
    for(i in 1:length(pathif[[1]]))
    {
      scores[,i]=pathif[[1]][[i]]
    }
  }
  
  # check that the pre-defined scores can be downloaded:
  if(method == "Pre-defined")
  {
    failed.to.load.predefined = FALSE
    # read pre-defined weights matrix:
    predefined.weights = read.csv(path.to.predefined.scores,row.names=1)
    # reset empty elements to 0
    predefined.weights[is.na(predefined.weights)]=0  
    
    # make sure that the pre-defined weights don't have genes outside the codeset.  if they do, give a warning:
    if(length(setdiff(dimnames(predefined.weights)[[1]],dimnames(normalized)[[2]]))>0)
    {
      warning(paste("Warning: The following genes in the predefined signatures are not included in the data: ",setdiff(dimnames(predefined.weights)[[1]],dimnames(normalized)[[2]])))
      cat(paste("LOG: Warning: The following genes in the predefined signatures are not included in the data: ",setdiff(dimnames(predefined.weights)[[1]],dimnames(normalized)[[2]])),file=log,sep='\n',append=TRUE)
      cat(paste("document.write('<p>Warning: The following genes in the predefined signatures are not included in the data: ",setdiff(dimnames(predefined.weights)[[1]],dimnames(normalized)[[2]]),"</p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
    }

    # match genes to columns of normalized data:
    predefined.weights = predefined.weights[match(dimnames(normalized)[[2]],dimnames(predefined.weights)[[1]]),]
    predefined.weights = as.matrix(predefined.weights)
    
    ## confirm the genes are ordered right in the final predefined weights matrix:
    if(sum(dimnames(predefined.weights)[[1]]!=dimnames(normalized)[[2]],na.rm=T)>0)
    {
      warning("Warning: Gene names in pre-defined weights do not match codeset gene names - pre-defined pathway scores cannot be calculated")
      cat("LOG: Warning: Gene names in pre-defined weights do not match codeset gene names - pre-defined pathway scores cannot be calculated",file=log,sep='\n\n',append=TRUE)
      cat("document.write('<p>Warning: Gene names in pre-defined weights do not match codeset gene names - pre-defined pathway scores cannot be calculated</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
      method = "none"
    }    
    if(!is.numeric(predefined.weights))
    {
      warning("Warning: Elements of pre-defined weights matrix are not all numeric - pre-defined pathway scores cannot be calculated")
      cat("LOG: Warning: Elements of pre-defined weights matrix are not all numeric - pre-defined pathway scores cannot be calculated",file=log,sep='\n\n',append=TRUE)
      cat("document.write('<p>Warning: Elements of pre-defined weights matrix are not all numeric - pre-defined pathway scores cannot be calculated</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
      method = "none"
    }
    # reset method to none if the loading failed:
    if(failed.to.load.predefined)
    {
      warning("Warning: Failed to load/parse .csv file for predefined scores - fitting pathway scores using principal component analysis (PCA)")
      cat("LOG: Warning: Failed to load/parse .csv file for predefined scores - fitting pathway scores using principal component analysis (PCA)",file=log,sep='\n\n',append=TRUE)
      cat("document.write('<p>Warning: Failed to load/parse .csv file for predefined scores - fitting pathway scores using principal component analysis (PCA)</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
      method = "none"
    }
  }

  # actually fit pre-defined scores:
  if(method == "Pre-defined")
  {
    print("Using Pre-defined")
    cat("LOG:Using Pre-defined",file=log,sep='\n\n',append=TRUE)
    cat("document.write('<p>Using Pre-defined</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
    intercepts = rep(0,dim(predefined.weights)[2])
    if(is.element("intercept",dimnames(predefined.weights)[[1]]))
    {
      intercepts = predefined.weights["intercept",]
      predefined.weights = predefined.weights[!is.element(dimnames(predefined.weights)[[1]],"intercept"),]
    }
    if(is.element("Intercept",dimnames(predefined.weights)[[1]]))
    {
      intercepts = predefined.weights["Intercept",]
      predefined.weights = predefined.weights[!is.element(dimnames(predefined.weights)[[1]],"Intercept"),]
    }
    scores = normalized %*% predefined.weights   
    for(i in 1:dim(scores)[2]){scores[,i] = scores[,i]+intercepts[i]}
    dimnames(scores)[[1]] = dimnames(normalized)[[1]]
    dimnames(scores)[[2]] = dimnames(predefined.weights)[[2]]
    write.csv(predefined.weights,file=paste(path.to.pathway.scores,"//pathway score gene weights.csv",sep=""))
  }
  
  # fit 1st PC-based pathway scores:
  if(method=="PC1")
  {
    print("Using 1st PC")
    cat("LOG:Using 1st PC",file=log,sep='\n\n',append=TRUE)
    cat("document.write('<p>Using 1st PC</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
    K = length(gene.set.names)
    scores = matrix(NA,dim(adjusted)[1],K)
    dimnames(scores)[[1]]=dimnames(adjusted)[[1]]
    dimnames(scores)[[2]]=gene.set.names
    weights = matrix(0,dim(adjusted)[2],K)
    dimnames(weights)[[1]]=dimnames(adjusted)[[2]]
    dimnames(weights)[[2]]=gene.set.names
    for(k in 1:K)
    {
      # isolate gene set data:
      tempgenes = dimnames(gene.sets)[[1]][gene.sets[,k]==1]
      temp = adjusted[,is.element(dimnames(adjusted)[[2]],tempgenes)]
      # remove least variable 10% of genes:
      genevars = apply(temp,2,var)
      lowvargenes = genevars<quantile(genevars,.1)
      highvargenes = setdiff(dimnames(temp)[[2]],lowvargenes)
      # run PCA, scaling each gene by its sd or by 0.1 (the sd due to noise), whichever is greater
      pc = prcomp(temp[,highvargenes],scale=pmax(apply(temp[,highvargenes],2,sd)),minsd)  # assuming 0.1 is the sd due to noise.
      #pc = prcomp(temp,scale=pmax(apply(temp,2,sd)),minsd)  # assuming 0.1 is the sd due to noise.
      
      weights[dimnames(temp)[[2]],k] = pc$rotation[,1]/pc$scale
      # calculate scores on unadjusted data:
      pc1 = adjusted%*%weights[,k]  
      # reorient if you have normal samples:
      if(length(is.normal)>0)
      {
        if(mean(pc1[is.normal],na.rm=T)>mean(pc1[!is.normal],na.rm=T)){pc1 = -pc1; weights[,k]=-weights[,k]}
      }
      # and save the result:
      scores[,k] = pc1
    }
    write.csv(weights,file=paste(path.to.pathway.scores,"//pathway score gene weights.csv",sep=""))
  }
  
  if(method!="none")
  {
    ### heatmap of pathway scores, colored by each covariate:
    # fill in row colors matrix:   
    rowcols = matrix("white",dim(scores)[1],1)
    cat(length(covariates),file=log,sep='\n',append=TRUE)
    cat(dim(covariates),file=log,sep='\n',append=TRUE)
    suppressWarnings(write.table(head(covariates),file=log,sep='\t',append=TRUE))
    if(dim(covariates)[2]>0)
    {
      rowcols = as.matrix(annotcols)
      write.csv(rowcols,file=paste(path.to.csvs,"//pathway scores - row colors for heatmaps.csv",sep=""))
    }
    cat("dim(rowcols):",file=log,sep='\n',append=TRUE)
    cat(dim(rowcols),file=log,sep='\n',append=TRUE)
    if(dim(rowcols)[2]==1){rowcols = cbind(rowcols,"white")}
  
    for(r in 1:length(plottypearg)){
      plottype=plottypearg[r];
      #tempfilename = drawplot(filename=paste(path.to.pathway.scores,"//pathway scores heatmap - by ",dimnames(covariates)[[2]][i],sep=""),
      tempfilename = drawplot(filename=paste(path.to.pathway.scores,"//pathway scores heatmap",sep=""),
                              plottype,height=2,heatmapres=TRUE)
	tempfilename=gsub(path.results,"results",tempfilename)

      par(mar=c(10,4,2,1))
      if(dim(covariates)[2]>0){ heatmap.plus(scale(scores),symm=FALSE,Colv=TRUE,RowSideColors=rowcols,margins=c(9,11),cexRow=1) }
      if(dim(covariates)[2]==0){ heatmap.plus(scale(scores),symm=FALSE,Colv=TRUE,margins=c(9,11),cexRow=1) }
    dev.off() } 
    
    ### plot correlation matrix:
    if(plot.cormatrix)
    {
      cormat = cor(scores)
      #rescale so min cor is -1; keep positive cors the same (do this so the positive and negative colors have comparable intensitites)
      neg.elements = sign(cormat)==-1
      cormat.rescaled = cormat/abs(min(cormat))
      cormat2 = replace(cormat,neg.elements,cormat.rescaled[neg.elements])
      for(r in 1:length(plottypearg)){
        plottype=plottypearg[r];
        tempfilename = drawplot(filename=paste(path.to.pathway.scores,"//pathway scores correlation heatmap",sep=""),plottype,width=1.2,height=1.2,heatmapres=TRUE)
	  tempfilename=gsub(path.results,"results",tempfilename)
      par(mar=c(10,4,2,1))
      hmcols<-colorRampPalette(c("blue","antiquewhite3",codecols2[4]))(256)
      hmcols = hmcols
      breaks = seq(-max(abs(cormat2)),max(abs(cormat2)),length.out=length(hmcols)+1)
      heatmap.plus(cormat2,col=hmcols,symm=TRUE,Colv="Rowv",breaks=breaks,margins=c(7,7))
      dev.off()}   
    }
    
    ## pairs plots of pathway scores:
    if(pairs.plot.of.scores)
    {
      npaths = dim(scores)[2]
      # assign colors if there are any covariates selected to plot against:
      pairscols = list(); pairscols[[1]]="black"
      covariate.names.for.pairsplots = c("")
      if(plot.against.covariates&(dim(covariates)[2]>0))
      {
        for(i in 1:dim(covariates)[2])
        {
          pairscols[[i]] = annotcols[,i]
          covariate.names.for.pairsplots[i] = dimnames(covariates)[[2]][i]
        }
      }
      # draw pairs plots:
      for(i in 1:length(pairscols))
      {
        for(k in 1:dim(scores)[2])
        {
          for(r in 1:length(plottypearg)){
            plottype=plottypearg[r];
          if(length(pairscols)>1){tempfilename = drawplot(paste(path.to.pathway.scores,"//","pathway scores pairs plot - ",dimnames(scores)[[2]][k]," - colored by ",covariate.names.for.pairsplots[i],sep=""),
                                           plottype,width = 1.5,height=2)}
          if(length(pairscols)==1){tempfilename = drawplot(paste(path.to.pathway.scores,"//","pathway scores pairs plot - ",dimnames(scores)[[2]][k],sep=""),
                                            plottype,width = 1.5,height=2)}
		tempfilename=gsub(path.results,"results",tempfilename)

          par(mfrow=c(4,3))
          par(mar = c(2.5,6,3,0))
          for(j in setdiff(1:dim(scores)[2],k))
          {
            plot(scores[,k],scores[,j],xlab=dimnames(scores)[[2]][k],ylab=dimnames(scores)[[2]][j],col=pairscols[[i]],cex.lab=2)
       	write.csv(pairscols[[i]],file=paste(path.to.csvs,"//pathway scores - pairs plot color",i,".csv",sep=""))   
            # draw legend
            if((j==3)&(dim(covariates)[2]>0))
            {
              if(covariate.type[i] == "categorical")
              {
                par(xpd=TRUE)
                legend("topright",inset=c(0,-1),col=annotcols2[[i]],legend=levels(covariates[,i]),pch=1)    
              }
              if(covariate.type[i] == "continuous")
              {
                par(xpd=TRUE)
                legend("topright",inset=c(0,-1),pch=c(NA,1,1),col=c("white",annotcols2[[i]][1],annotcols2[[i]][3]),legend=c(covariate.names.for.pairsplots[i],"low","high"))
              }
            }
          }
          dev.off()}
        }
      }
    }
    ## pairs plots of PCs fit to pathway scores:
    if(PC.pairs.plot.of.scores)
    {
      pc = prcomp(scores,scale=TRUE)
      # assign colors if there are any covariates selected to plot against:
      pairscols = list(); pairscols[[1]]="black"
      covariate.names.for.pairsplots = c("")
      if(plot.against.covariates&(dim(covariates)[2]>0))
      {
        for(i in 1:dim(covariates)[2])
        {
          pairscols[[i]] = annotcols[,i]
          covariate.names.for.pairsplots[i] = dimnames(covariates)[[2]][i]
        }
      }

print("Creating pathway score HTML infrastructure")
cat("LOG:Creating pathway score HTML infrastructure",file=log,sep='\n\n',append=TRUE)
cat("document.write('<p>Creating pathway score HTML infrastructure</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)

	if(plot.against.covariates&(dim(covariates)[2]>0))
	{
	#do we need to handle this case?
	}else{
	cat("document.write('              	    	<li>PC Fit to Pathway Scores<br><img src=\"results/Pathway scoring/principal components fit to pathway scores pairs plot.png\"></li>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
	}
	 cat("document.write('              	    				<li>Links To Additional Data<br>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
	 cat("document.write('        			                 <ul>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)

	 strTemp=paste("document.write('        			                   <li><a href=\"results//Pathway scoring//pathway scores.csv","\">Pathway Scores</a></li>')\n",sep="")
	 cat(strTemp,file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)

      if(deregmodule.arg.method!="Pathifier")
      {
	 strTemp=paste("document.write('        			                   <li><a href=\"results//Pathway scoring//pathway score gene weights.csv","\">Pathway Score Gene Weights</a></li>')\n",sep="")
	 cat(strTemp,file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
      } 
	 strTemp=paste("document.write('        			                   <li><a href=\"results//Pathway scoring//p-values of pathway scores vs. covariates.csv","\">P-values of Pathway Scores vs Covariates</a></li>')\n",sep="")
	 cat(strTemp,file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)


	 cat("document.write('        			                 </ul>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)

	 cat("document.write('        			          </ul>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
	 cat("document.write('                    </div>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)


#####PC plots menu entries
if(plot.against.covariates&(dim(covariates)[2]>0))
{
cat("document.write('	 			<ul id=\"PSPCA\" class=\"f-dropdown\" data-dropdown-content data-tab>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)


      for(i in 1:length(pairscols))
      {
		catTemp <-covariate.names.for.pairsplots[i]
		catTemp = gsub("\\.","",catTemp)
		catTemp = gsub("\\s","",catTemp)	
	 	strTemp=paste("document.write('	 			  <li><a href=\"#summary-pathscore-PCP-",catTemp,"\">",covariate.names.for.pairsplots[i],"</a></li>');\n",sep="")
	 	cat(strTemp,file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
 
      }

cat("document.write('	 			</ul>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)

}


##pathscore -

      # draw pairs plots:
      for(i in 1:length(pairscols))
      {

	   catTemp <-covariate.names.for.pairsplots[i]
        catTemp = gsub("\\.","",catTemp)
    	   catTemp = gsub("\\s","",catTemp)
        for(r in 1:length(plottypearg))
       {
          plottype=plottypearg[r];

          if(length(pairscols)>1){tempfilename = drawplot(paste(path.to.pathway.scores,"//","principal components fit to pathway scores - pairs plot colored by ",covariate.names.for.pairsplots[i],sep=""),
                                         plottype)}
          if(length(pairscols)==1){tempfilename = drawplot(paste(path.to.pathway.scores,"//","principal components fit to pathway scores pairs plot",sep=""),
                                          plottype)}
		tempfilename=gsub(path.results,"results",tempfilename)
          pairs(pc$x[,1:4],col=pairscols[[i]],xaxt="n",yaxt="n")
    	     write.csv(pairscols[[i]],file=paste(path.to.csvs,"//pathway scores - PC pairs plot color",i,".csv",sep=""))   
          dev.off()
		if(r==1)
		{
			#need to revisit - can't include this file in this scope....
			if(length(pairscols)>1)
			{
		 	strTemp=paste("document.write('	 	    <div class=\"content\" id=\"summary-pathscore-PCP-",catTemp,"\">');\n",sep="")
			cat(strTemp,file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
			cat("document.write('              	    		  	<ul class=\"small-block-grid-1\">');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)

			#add a div for each
			strTemp=paste("document.write('              	    				<li>PC Fit to Pathway Score - ",covariate.names.for.pairsplots[i],"<br><img src=\"",tempfilename,".png\"></img><img src=\"",tempfilename," - legend.png\"></img></li>');\n",sep="")
			cat(strTemp,file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
		     cat("document.write('        	  	          	     </ul>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
      	     cat("document.write('                    </div>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
			} else {
		 	strTemp=paste("document.write('	 	    <div class=\"content\" id=\"summary-pathscore-PCP-",catTemp,"\">');\n",sep="")
			cat(strTemp,file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)

			cat("document.write('              	    		  	<ul class=\"small-block-grid-1\">');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
			strTemp=paste("document.write('              	    				<li>PC Fit to Pathway Score<br><img src=\"",tempfilename,".png\"></img></li>');\n",sep="")
			cat(strTemp,file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
			cat("document.write('        	  	          	     </ul>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
      	     cat("document.write('                    </div>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)

			}
		}

        }
        if(dim(covariates)[2]>0)
        {
          for(r in 1:length(plottypearg)){
            plottype=plottypearg[r];
            
            if(length(pairscols)>1){tempfilename = drawplot(paste(path.to.pathway.scores,"//","principal components fit to pathway scores - pairs plot colored by ",covariate.names.for.pairsplots[i]," - legend",sep=""),
                                             plottype,width=.5)}
            if(length(pairscols)==1){tempfilename = drawplot(paste(path.to.pathway.scores,"//","principal components fit to pathway scores pairs plot - legend",sep=""),
                                              plottype,width=.5)}
		tempfilename=gsub(path.results,"results",tempfilename)

          par(mar = rep(0,4))
          plot(0,0,col=0,bty ="n",axes=F,frame.plot=F, xaxt='n',yaxt="n",xlab="",ylab="")
          if(covariate.type[i] == "categorical")
          {
            par(xpd=TRUE)
            legend("center",col=c(NA,annotcols2[[i]]),legend=c(covariate.names.for.pairsplots[i],levels(covariates[,i])),pch=c(NA,rep(1,length(unique(covariates[,i])))))    
          }
          if(covariate.type[i] == "continuous")
          {
            par(xpd=TRUE)
            legend("center",pch=c(NA,1,1),col=c("white",annotcols2[[i]][1],annotcols2[[i]][3]),legend=c(covariate.names.for.pairsplots[i],"low","high"))
          }
          dev.off()}
        }
      }
    }

    	cat("document.write('                    </div>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
    	cat("document.write('                    </div>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)

#end of section

    ## plot scores by covariates and save univariate pvals:
    pvals=NULL
    npaths = dim(scores)[2]

 # loop through pathways:	
    for(j in 1:npaths)
    {
    if(plot.against.covariates&(dim(covariates)[2]>0))
    {
      
      # matrix of pvalues vs covariates:
      pvals = matrix(NA,npaths,dim(covariates)[2])
      dimnames(pvals)[[1]] = dimnames(scores)[[2]]
      dimnames(pvals)[[2]] = dimnames(covariates)[[2]]
     

	strTemp=paste("document.write('                       <div class=\"content\" id=\"",dimnames(scores)[[2]][j],"-pathscore\">');\n",sep="")
	cat(strTemp,file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)

	cat("document.write('	 			   <nav class=\"top-bar\" data-topbar>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
	cat("document.write('	 			         <ul class=\"button-group  tabs\" data-tab>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)

	# first loop to create menu
      for(i in 1:dim(covariates)[2])
      {
		catTemp <-names(covariates)[i]
		catTemp = gsub("\\.","",catTemp)
		catTemp = gsub("\\s","",catTemp)
		
		###return to here
		if(i==1)
		{
			strTemp=paste("document.write('						   <li class=\"tab-title active\"><a href=\"#",dimnames(scores)[[2]][j],"-pathscore-",catTemp,"\">",names(covariates)[i],"</a></li>');\n",sep="")

		}else{
			strTemp=paste("document.write('						   <li class=\"tab-title\"><a href=\"#",dimnames(scores)[[2]][j],"-pathscore-",catTemp,"\">",names(covariates)[i],"</a></li>');\n",sep="")
		}	
		cat(strTemp,file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)

	 }
	cat("document.write('	 			         </ul>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
	cat("document.write('	 			   </nav>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
	## end NAV menu for pathway - start div for pathway content
	cat("document.write('		          <div class=\"tabs-content\">');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)

	#loop through covariates
      for(i in 1:dim(covariates)[2])
      {
		catTemp <-names(covariates)[i]
		catTemp = gsub("\\.","",catTemp)
		catTemp = gsub("\\s","",catTemp)
		###make first covariate default 
		if(i==1)
		{
			strTemp=paste("document.write('		          <div class=\"content active\" id=\"",dimnames(scores)[[2]][j],"-pathscore-",catTemp,"\">');\n",sep="")

		}else{
			strTemp=paste("document.write('		          <div class=\"content \" id=\"",dimnames(scores)[[2]][j],"-pathscore-",catTemp,"\">');\n",sep="")
		}
		cat(strTemp,file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
		cat("document.write('              	    		  	<ul class=\"small-block-grid-1\">');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)



        # ID type of covariate
        if(covariate.type[i] == "continuous")
        {
            for(r in 1:length(plottypearg)){
            plottype=plottypearg[r];
            tempfilename = drawplot(paste(path.to.pathway.scores,"//",names(covariates)[i],dimnames(scores)[[2]][j],sep=""),plottype)
		 tempfilename=gsub(path.results,"results",tempfilename)
	   	if(r==1)
	   	{		 
			strTemp=paste("document.write('              	    		  	   <li>Pathway Score vs ",names(covariates)[i],"<br><img src=\"",tempfilename,".png\"></li>');\n",sep="")
		      cat(strTemp,file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
		}
            plot(scores[,j]~covariates[,i],xlab = dimnames(covariates)[[2]][i],ylab=paste(dimnames(scores)[[2]][j],"pathway score"),cex.lab=1.5)
            dev.off()}
            # and run hypothesis test:
            pvals[j,i] = summary(lm(scores[,j]~covariates[,i]))$coefficients[2,4]
        }
        if(covariate.type[i] == "categorical")
        {
          x = as.factor(covariates[,i])
            y = as.vector(scores[,j])
            for(r in 1:length(plottypearg)){
              plottype=plottypearg[r];
              tempfilename = drawplot(paste(path.to.pathway.scores,"//",names(covariates)[i],dimnames(scores)[[2]][j],sep=""),plottype)
		 tempfilename=gsub(path.results,"results",tempfilename)
	     if(r==1)
		{  
		 strTemp=paste("document.write('              	    		  	   <li>Pathway Score vs ",names(covariates)[i],"<br><img src=\"",tempfilename,".png\"></li>');\n",sep="")
		 cat(strTemp,file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
		}
            par(mar=c(10,6,2,1))
            boxplot(y~x,border="darkgrey",ylab=paste(dimnames(scores)[[2]][j],"pathway score"),outline=FALSE,las=2,cex.lab=1.5)
		        points(y~jitter(as.numeric(x)),col=annotcols2[[i]][x],pch=16)
            dev.off()}
            # and run hypothesis test:
            complete = !is.na(covariates[,i])
            pvals[j,i] = anova(lm(scores[complete,j]~covariates[complete,i]),lm(scores[complete,j]~1))[[6]][2]

        }
	if(length(pairscols)>1)
	{
      	strTemp=paste("document.write('              	    		  	   <li>Pairs Plot Colored By ",names(covariates)[i],"<br><img src=\"results/Pathway scoring/pathway scores pairs plot - ",dimnames(scores)[[2]][j]," - colored by ",names(covariates)[i],".png\"></li>');\n",sep="")
	}else{
		strTemp=paste("document.write('              	    		  	   <li>Pairs Plot<br><img src=\"results/Pathway scoring/pathway scores pairs plot - ",dimnames(scores)[[2]][j],".png\"></li>');\n",sep="")
	}
	cat(strTemp,file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
	cat("document.write('              	    			</ul>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
	cat("document.write('                    </div>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
      }
	#close pathway div
	cat("document.write('	 		</div>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
	cat("document.write('	 		</div>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
	}
	else
	{
		strTemp=paste("document.write('                       <div class=\"content\" id=\"",pathwaynames[j],"-pathscore\">');\n",sep="")
		cat(strTemp,file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
		cat("document.write('              	    		  	<ul class=\"small-block-grid-1\">');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
	strTemp=paste("document.write('              	    		  	   <li>Pairs Plot - ",pathwaynames[j],"<br><img src=\"results/Pathway scoring/pathway scores pairs plot - ",pathwaynames[j],".png\"></li>');\n",sep="")
		cat(strTemp,file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
		cat("document.write('              	    		  	</ul>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
		cat("document.write('                       </div>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)

	}
     }
  	write.csv(covariates,file=paste(path.to.csvs,"//pathway scores - covariates matrix used in plots against covariates.csv",sep=""))
     write.csv(pvals,file=paste(path.to.pathway.scores,"//p-values of pathway scores vs. covariates.csv",sep=""))



    #}#end of if statement in question
#end of HTML file
 cat("document.write('                    </div>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)
 cat("document.write('                    </div>');\n",file=paste(path.inc,"//panel2_1.js",sep=""),append=TRUE)


    ## heatmap of associations between scores and covariates: if scores are directed, look at association.  if undirected, look at p-values:
    write.csv(scores,file=paste(path.to.pathway.scores,"//pathway scores.csv",sep=""))
print("Finished pathway scoring")
cat("LOG:Finished pathway scoring",file=log,sep='\n\n',append=TRUE)
cat("document.write('<p>Finished pathway scoring</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
    return(list(scores=scores,pvals=pvals))

  } #end if(method!="none")
  if(method=="none"){return(scores=NULL)}



}
  

#################################
#################################
#   Differential Expression (DE) analyses
#################################
#################################

# fit one model with all chosen covariates, then report results separately for each variable of interest:

# normalized: normalized data matrix
# predictors: names of predictors in annot data frame
# confounders: names of confounders in annot data frame
# annot: annotation data frame
# predictor.var.type: vector of "cont","cat" 
# confounder.var.type: vector of "cont","cat","surv"
run.DE.analyses = function(normalized,predictors,confounders,predictor.var.type,annot,
                           confounder.var.type,pval.adjustment = c("BY","BH","Bonf","none"),gene.sets=NULL,saveresults=TRUE)
{
print("Starting differential expression (DE) analysis")
cat("LOG:Starting differential expression (DE) analysis",file=log,sep='\n\n',append=TRUE)
cat("document.write('<p>Starting differential expression (DE) analysis</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
saveresults1 = "genes"
if(!saveresults){saveresults1="pathways"}
  covariates = as.data.frame(annot[,c(predictors,confounders)])
  names(covariates) = c(predictors,confounders)
  predictor = c(rep(TRUE,length(predictors)),rep(FALSE,length(confounders))) # vector indicating which variable is a predictor
  cat("predictor variable:",file=log,sep='\n',append=TRUE)
  suppressWarnings(write.table(predictor,file=log,sep='\t',append=TRUE))
  ### evaluate missingness
  missingobs = sum(rowSums(is.na(covariates))>0)
  missingpervariable = colSums(is.na(covariates))
  line1 = paste("There are",missingobs,"samples missing data from at least one covariate.  They will be removed from the differential expression analysis.")
  line2 = ""
  for(i in 1:length(missingpervariable))
  {
    line2 = paste(line2,paste("\n",dimnames(covariates)[[2]][i],"is missing",missingpervariable[i],"observations.  They will be removed from the differential expression analysis."),sep="")
  }
  write.table(paste(line1,"\n",line2),file=paste(path.to.DE.results,"//Missing data warning.txt",sep=""),
              row.names=FALSE,col.names=FALSE)
  write.csv(covariates,file=paste(path.to.csvs,"//DE - covariates",saveresults1,".csv",sep=""))  
 
  # for each predictor: log fold change (named by contrast), pval, se
  temp1 = normalized[,1]
  temp = cbind(temp1,covariates)
  form = paste("temp1~")
  for(k in 1:dim(covariates)[2]){form = paste(form,"+",dimnames(covariates)[[2]][k])}
  # enter the formula in lm():
  lm1 = lm(form,data=temp)
  ests = matrix(NA,dim(normalized)[2],dim(summary(lm1)$coefficients)[1]-1)
  dimnames(ests)[[1]] = dimnames(normalized)[[2]]; dimnames(ests)[[2]] = dimnames(summary(lm1)$coefficients)[[1]][-1]

  ses = pvals = ests
  active.variables=as.list(dimnames(ests)[[2]])
  #print(paste("active vars:",active.variables,sep=""))
  Pathways = c()
  for(j in 1:dim(ests)[1])
  {
    # define formula:
    temp1=normalized[,j]
    temp = cbind(temp1,covariates)
    form = paste("temp1~")
    for(k in 1:dim(covariates)[2]){form = paste(form,"+",dimnames(covariates)[[2]][k])}
    # enter the formula in lm():
    lm1 = lm(form,data=temp)
    mod = summary(lm1)$coefficients[-1,]
    if(is.vector(mod)){mod = t(as.matrix(mod))}
       
    ests[j,] = mod[,1]
    ses[j,] = mod[,2]
    pvals[j,] = mod[,4]
    # and get pathway memberships:
    if(length(gene.sets)>0)
    {
      Pathways[j] = paste(dimnames(gene.sets)[[2]][gene.sets[dimnames(ests)[[1]][j],]==1],collapse=", ")
    }
  }
  
  ## now identify the columns corresponding to predictors vs. confounders, removing columns  that lm() has disregarded due to linear dependency:
  npredictors=0
  cat("predictor.var.type:",file=log,sep='\n',append=TRUE)
  suppressWarnings(write.table(predictor.var.type,file=log,sep='\t',append=TRUE))

  for(i in 1:sum(predictor))
  {
  
    suppressWarnings(write.table(dimnames(covariates)[[2]][i],file=log,sep='\t',append=TRUE))
    cat("dimnames(covariates)[[2]][i]:",file=log,sep='\n',append=TRUE)
    cat(dimnames(covariates)[[2]][i],file=log,sep='\n',append=TRUE)
    cat(nchar(dimnames(covariates)[[2]][i]),file=log,sep='\n',append=TRUE)
    cat(nchar(dimnames(covariates)[[2]][i]),file=log,sep='\n',append=TRUE)
    short=lapply(active.variables, function(x) substr(x,1,nchar(dimnames(covariates)[[2]][i])))
    cat("short:",file=log,sep='\n',append=TRUE)
    suppressWarnings(write.table(short,file=log,sep='\n',append=TRUE))
    cat("active.levels:",file=log,sep='\n',append=TRUE)
    cat(length(short[short == dimnames(covariates)[[2]][i]]),file=log,sep='\n',append=TRUE)
  
    if(predictor.var.type[i]=="categorical"){
      if(length(short[short == dimnames(covariates)[[2]][i]])<1){
        warning(paste("Warning: ",dimnames(covariates)[[2]][i]," is collinear with another covariate in the data matrix.  It will be removed from all subsequent analyses.", sep=""))
        cat(paste("LOG:Warning: ",dimnames(covariates)[[2]][i]," is collinear with another covariate in the data matrix.  It will be removed from all subsequent analyses.", sep=""),file=log,sep='\n',append=TRUE)
        cat(paste("document.write('<p>Warning: ",dimnames(covariates)[[2]][i]," is collinear with another covariate in the data matrix.  It will be removed from all subsequent analyses.</p>');",sep=""), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
      }else if(length(short[short == dimnames(covariates)[[2]][i]])<(length(levels(covariates[,i]))-1)){
        warning(paste("Warning: ",dimnames(covariates)[[2]][i]," has levels that are collinear with another covariate in the data matrix.  They will be removed from all subsequent analyses.", sep=""))
        cat(paste("LOG:Warning: ",dimnames(covariates)[[2]][i]," has levels that are collinear with another covariate in the data matrix.  They will be removed from all subsequent analyses.", sep=""),file=log,sep='\n',append=TRUE)
        cat(paste("document.write('<p>Warning: ",dimnames(covariates)[[2]][i]," has levels that are collinear with another covariate in the data matrix.  They will be removed from all subsequent analyses.</p>');",sep=""), file=paste(path.inc,"//status.js",sep=""),append=TRUE)   
      }else{
        npredictors=npredictors+length(short[short == dimnames(covariates)[[2]][i]])
        cat("npredictors:",file=log,sep='\n',append=TRUE)
        cat(npredictors,file=log,sep='\n',append=TRUE)
      }
    }
    if(predictor.var.type[i]=="continuous"){
      if(length(short[short == dimnames(covariates)[[2]][i]])<1){
        warning(paste("Warning: ",dimnames(covariates)[[2]][i]," is collinear with another covariate in the data matrix.  It will be removed from all subsequent analyses.", sep=""))
        cat(paste("LOG:Warning: ",dimnames(covariates)[[2]][i]," is collinear with another covariate in the data matrix.  It will be removed from all subsequent analyses.", sep=""),file=log,sep='\n',append=TRUE)
        cat(paste("document.write('<p>Warning: ",dimnames(covariates)[[2]][i]," is collinear with another covariate in the data matrix.  It will be removed from all subsequent analyses.</p>');",sep=""), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
      }else{
        npredictors=npredictors+1
        cat("npredictors:",file=log,sep='\n',append=TRUE)
        cat(npredictors,file=log,sep='\n',append=TRUE)
      }
    }
  }

  if(saveresults)  
  {    
    cat("npredictors:",file=log,sep='\n',append=TRUE)
    suppressWarnings(write.table(npredictors,file=log,sep='\t',append=TRUE))
    cat("head(pvals):",file=log,sep='\n',append=TRUE)
    suppressWarnings(write.table(head(pvals),file=log,sep='\t',append=TRUE))
	#--- START HTML report code injection---#
	print("Creating differential expression (DE) HTML infrastructure")
     cat("LOG:Creating differential expression (DE) HTML infrastructure",file=log,sep='\n\n',append=TRUE)
	cat("document.write('<p>Creating differential expression (DE) HTML infrastructure</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
  
  # create DE include file 
	cat("document.write('		<div class=\"large-12 columns\">');\n",file=paste(path.inc,"//panel2_2.js",sep=""),append=TRUE)
	cat("document.write('	 	  <nav class=\"top-bar\" data-topbar>');\n",file=paste(path.inc,"//panel2_2.js",sep=""),append=TRUE)
	cat("document.write('	 		<ul class=\"button-group  tabs\" data-tab>');\n",file=paste(path.inc,"//panel2_2.js",sep=""),append=TRUE)
   

##DEMENU
  catlevels2<-dimnames(pvals)[[2]]
  catlevels<-catlevels2[1:npredictors]	

  for(i in 1:length(predictors))
  {
		kk=i
		preTemp<-predictors[i]
		
		numlevels=0
		tempLevels<-c()
		tempLevelNames<-c()
		#need to check how many levels a covariate has, and also check for 
		for(j in 1:length(catlevels))	
		{
			test1 = gsub(preTemp,"",catlevels[j])
			if(is.element(paste(preTemp,test1,sep=""),catlevels))
			{
				#is a valid catlevel 
				numlevels = numlevels+1
				
				testNumeric=paste(catlevels[j],sep="")
				y<-charToRaw(testNumeric)
				stringNumeric=paste(y, collapse = '')

				tempLevels<-c(tempLevels,stringNumeric)
				tempLevelNames<-c(tempLevelNames,test1)
				
			}
		}
		

		lvlTemp <-tempLevels[1]


		#decrement the 'active' counter if covariate is not valid
		if(numlevels==0)
		{
		  kk=kk-1
		}
		if(numlevels==1)#continuous
		{
		if(kk==1)
		{
			strTemp=paste("document.write('		                 <li class=\"tab-title active\"><a href=\"#DE-",lvlTemp,"\">",predictors[i],"</a></li>');\n",sep="")
		}else{
			strTemp=paste("document.write('		                 <li class=\"tab-title\"><a href=\"#DE-",lvlTemp,"\">",predictors[i],"</a></li>');\n",sep="")
		}
		cat(strTemp,file=paste(path.inc,"//panel2_2.js",sep=""),append=TRUE)
		}
		if(numlevels>1)#categorical
		{
			#add top level menu the submenu

			preTemp = gsub("\\.","",preTemp)
			preTemp = gsub("\\s","",preTemp)
		if(kk==1)
		{
			strTemp=paste("document.write('		                 <li class=\"tab-title active\"><a href=\"#DE-",preTemp,"\" data-dropdown=\"dd",preTemp,"\">",predictors[i],"</a></li>');\n",sep="")
		}else{
			strTemp=paste("document.write('		                 <li class=\"tab-title\"><a href=\"#DE-",preTemp,"\" data-dropdown=\"dd",preTemp,"\">",predictors[i],"</a></li>');\n",sep="")
			}
			cat(strTemp,file=paste(path.inc,"//panel2_2.js",sep=""),append=TRUE)	

			strTemp=paste("document.write('	  	                   <ul id=\"dd",preTemp,"\" class=\"f-dropdown\" data-dropdown-content data-tab>');\n",sep="")
			cat(strTemp,file=paste(path.inc,"//panel2_2.js",sep=""),append=TRUE)	

			for(lvl in 1:numlevels)
			{

				lvlTemp <-tempLevels[lvl]

				strTemp=paste("document.write('	    	                     <li><a href=\"#DE-",lvlTemp,"\">",tempLevelNames[lvl],"</a></li>');\n",sep="")
				cat(strTemp,file=paste(path.inc,"//panel2_2.js",sep=""),append=TRUE)	

			}
			cat("document.write('	  	                   </ul>');\n",file=paste(path.inc,"//panel2_2.js",sep=""),append=TRUE)
		}
  }

	cat("document.write('	 		   </ul>');\n",file=paste(path.inc,"//panel2_2.js",sep=""),append=TRUE)
	cat("document.write('	 	  </nav>');\n",file=paste(path.inc,"//panel2_2.js",sep=""),append=TRUE)
#END OF MENU	
	cat("document.write('          <div class=\"tabs-content\">');\n",file=paste(path.inc,"//panel2_2.js",sep=""),append=TRUE)

    # volcano plot, coloring by significance level and naming top 100 genes
    for(i in 1:npredictors)
    {
	catTemp <-dimnames(pvals)[[2]][i]
	y<-charToRaw(catTemp)
	strNumeric=paste(y, collapse = '')

	if(i==1)
	{
 		strTemp=paste("document.write('             <div class=\"content active\" id=\"DE-",strNumeric,"\">');\n",sep="")
	}else{
		strTemp=paste("document.write('             <div class=\"content\" id=\"DE-",strNumeric,"\">');\n",sep="")
	}
	cat(strTemp,file=paste(path.inc,"//panel2_2.js",sep=""),append=TRUE)
	cat("document.write('		  	<ul class=\"small-block-grid-2\">');\n",file=paste(path.inc,"//panel2_2.js",sep=""),append=TRUE)
	
      volcanocol = rep("black",dim(pvals)[1])
      volcanocol[pvals[,i]<0.01]=codecols2[1]
      volcanocol[pvals[,i]<0.001]=codecols2[2]
      volcanopch = rep(1,dim(pvals)[1])
      volcanopch[pvals[,i]<0.01]=16
      volcanopch[pvals[,i]<0.001]=16

      for(r in 1:length(plottypearg))
      {
	   plottype=plottypearg[r];
        tempfilename = drawplot(paste(path.to.DE.results,"//volcano plot",dimnames(pvals)[[2]][i],sep=""),plottype,width=1.3,height=1.3)
	   tempfilename=gsub(path.results,"results",tempfilename)
	   if(r==1)
	   {
          strTemp=paste("document.write('		  	  <li>Volcano Plot - ",dimnames(pvals)[[2]][i],"<br><img src=\"",tempfilename,".png\"></img></li>');\n",sep="")
	     cat(strTemp,file=paste(path.inc,"//panel2_2.js",sep=""),append=TRUE)
	   }
        plot(ests[,i],-log10(pvals[,i]),xlab="Log(fold change)",ylab = "-log10(p-value)",main=dimnames(pvals)[[2]][i],col=volcanocol,pch=volcanopch)   
        abline(h=-log10(c(0.01,0.001)),lty=2)
        legend("bottomright",pch=c(16,16),col=c(codecols2[1],codecols2[2]),legend = c("p < 0.01","p < 0.001"))
        top40 = order(pvals[,i],decreasing=FALSE)[1:40]
        text(x=ests[top40,i],y=-log10(pvals[top40,i]),labels=dimnames(ests)[[1]][top40],cex=.8)
        dev.off()
      }
  
    # write output tables for predictors
      out = cbind(ests[,i],ests[,i]-1.96*ses[,i],ests[,i]+1.96*ses[,i],pvals[,i])
      dimnames(out)[[2]] = c("Log fold change","Lower confidence limit","Upper confidence limit","P-value")
	    extracolumn=FALSE
	    if(pval.adjustment=="BY")
      {
        extracolumn=TRUE
        pvals.a = p.adjust(pvals[,i],method="BY")
        out = cbind(out,pvals.a); dimnames(out)[[2]][5] = "FDR"
      }
      if(pval.adjustment=="BH")
      {
        extracolumn=TRUE
        pvals.a = p.adjust(pvals[,i],method="BH")
        out = cbind(out,pvals.a); dimnames(out)[[2]][5] = "FDR"
      }
      if(pval.adjustment=="Bonferroni")
      {
        extracolumn=TRUE
        pvals.a = p.adjust(pvals[,i],method="bonferroni")
        out = cbind(out,pvals.a); dimnames(out)[[2]][5] = "Bonf. p-value"
      }
      out = cbind(as.data.frame(out),Pathways)
      write.csv(out,file=correct.filename(paste(path.to.DE.results,"//DE results - ",dimnames(ests)[[2]][i],".csv",sep="")))  

      # and write a top hits version:
      out2 = out[order(pvals[,i])[1:20],]
      out2[,1:3]=signif(out2[,1:3],3)
      out2[,4]=signif(out2[,4],4)
      if(extracolumn){out2[,5]=signif(out2[,5],4)}
    
      write.csv(out2,file=correct.filename(paste(path.to.DE.results,"//DE results - Top 20 - ",dimnames(ests)[[2]][i],".csv",sep="")))

	 #HTML placeholder for top20 table
      #strTemp=paste("document.write('		<li>DE results - Top 20 - ",dimnames(ests)[[2]][i],"<br>table placeholder</li>');\n",sep="")
	 #cat(strTemp,file=paste(path.inc,"//panel2_2.js",sep=""),append=TRUE)
	 #END HTML

#START HTML TABLE
numRow=nrow(out2)
numCol=ncol(out2)

genes=rownames(out2)	
cols=dimnames(out2)[[2]]


#output HTML tables here - 
	 strTemp=paste("document.write('              	    				<li>DE results - Top 20 - ",dimnames(ests)[[2]][i],"<br>');\n",sep="")
	 cat(strTemp,file=paste(path.inc,"//panel2_2.js",sep=""),append=TRUE)

	  if(numCol==6)
	  {
	  	outtable=xtable(out2,display=c("s","g","g","g","g","g","s"))
	  }else{
		outtable=xtable(out2,display=c("s","g","g","g","g","s"))
	  }
        tabletext=print(outtable,type="html",file="",print.results=FALSE,timestamp=NULL,comment=FALSE,html.table.attributes="")
	   	   
	  tabletext=gsub("<table >","document.write('<table>');\n",tabletext)
	  tabletext=gsub("<tr>","document.write('<tr>",tabletext)
	  tabletext=gsub("</tr>","</tr>');\n",tabletext)
	  tabletext=gsub("</table>","document.write('</table>');\n",tabletext)
	  cat(tabletext,file=paste(path.inc,"//panel2_2.js",sep=""),append=TRUE)

	 cat("document.write('                        </li>')\n",file=paste(path.inc,"//panel2_2.js",sep=""),append=TRUE)
#END HTML table

  	cat("document.write('		  	</ul>');\n",file=paste(path.inc,"//panel2_2.js",sep=""),append=TRUE)
  	cat("document.write('		</div>');\n",file=paste(path.inc,"//panel2_2.js",sep=""),append=TRUE)  


      
    }
cat("document.write('	   </div>');\n",file=paste(path.inc,"//panel2_2.js",sep=""),append=TRUE)  
cat("document.write('	  </div>');\n",file=paste(path.inc,"//panel2_2.js",sep=""),append=TRUE)  


  } # end if(saveresults)

print("Creating differential expression (DE) analysis results files")
cat("LOG:Creating differential expression (DE) analysis results files",file=log,sep='\n\n',append=TRUE)
cat("document.write('<p>Creating differential expression (DE) analysis results files</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)

# tweak covariate names to avoid illegal characters:
write.csv(ests,file=paste(path.to.csvs,"//DE results - ests",saveresults1,".csv",sep=""))
write.csv(ses,file=paste(path.to.csvs,"//DE results - ses",saveresults1,".csv",sep=""))
write.csv(pvals,file=paste(path.to.csvs,"//DE results - pvals",saveresults1,".csv",sep=""))
  dimnames(ests)[[2]] = gsub("/",".",dimnames(ests)[[2]])
  dimnames(pvals)[[2]] = dimnames(ests)[[2]]
  dimnames(ses)[[2]] = dimnames(ests)[[2]]
  out = list(ests=ests,ses=ses,pvals=pvals,npredictors=npredictors,Pathways=Pathways)
  

print("Finished differential expression (DE) analysis")
cat("LOG:Finished differential expression (DE) analysis",file=log,sep='\n\n',append=TRUE)
cat("document.write('<p>Finished differential expression (DE) analysis</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
return(out)
}

#################################
#################################
## Gene set analysis module
#################################
#################################
run.gene.set.analyses = function(DEresults,gene.sets,gene.set.names,deregscores=NULL,saveresults=TRUE,pval.adjustment,predictors)
{
  print ("Starting gene set analysis (GSA)")
  cat("LOG:Starting gene set analysis (GSA)",file=log,sep='\n\n',append=TRUE)
  cat("document.write('<p>Starting gene set analysis (GSA)</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
  nvars = DEresults$npredictors
  pvals=as.data.frame(DEresults$pvals[,1:nvars]); dimnames(pvals)[[2]]=dimnames(DEresults$pvals)[[2]][1:nvars]
  ests=as.data.frame(DEresults$ests[,1:nvars]); dimnames(ests)[[2]]=dimnames(DEresults$ests)[[2]][1:nvars]
  ngenes = dim(DEresults$pvals)[1]
  # revise gene.sets so the genes match those of DEresults:
  gene.sets = gene.sets[match(dimnames(pvals)[[1]],dimnames(gene.sets)[[1]]),]
  K = dim(gene.sets)[2]
  covariates = dimnames(DEresults$ests)[[2]][1:nvars]

  # matrices of statistics to save:
  global.sig.stats = matrix(NA,dim(gene.sets)[2],length(covariates))
  dimnames(global.sig.stats)[[1]]=gene.set.names
  dimnames(global.sig.stats)[[2]]=covariates
  global.sig.stats.updown = global.sig.stats

#BEGIN HTML
#write GSA include file
#static content - left menu
cat("document.write('	    <div class=\"large-2 columns\" id=\"side_menu\">');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
cat("document.write('    		  <H4>Pathways</H4>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
cat("document.write('              <ul class=\"button-group tabs vertical\" data-tab>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title active\"><a href=\"#summary-GSA\"><b>Summary</b></a></li>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#Apop-GSA\">Apoptosis</a></li>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#CC-GSA\">Cell Cycle</a></li>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#ChromMod-GSA\">Chromatin Modification</a></li>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
cat("document.write('			     <li class=\"tab-title\"><a href=\"#DNARepair-GSA\">DNA Repair</a></li>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#HH-GSA\">Hedgehog</a></li>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#STAT-GSA\">JAK-STAT</a></li>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#MAPK-GSA\">MAPK</a></li>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#Notch-GSA\">Notch</a></li>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#PI3K-GSA\">PI3K</a></li>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#RAS-GSA\">RAS</a></li>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#TGFB-GSA\">TGF-Beta</a></li>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#TXmisReg-GSA\">TX Mis Regulation</a></li>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#Wnt-GSA\">Wnt</a></li>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)

cat("document.write('              </ul>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
cat("document.write('          </div>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
cat("document.write('		 <div class=\"large-10 columns\">');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
cat("document.write('		   <div class=\"tabs-content\">');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)


#END STATIC HTML

for(k in 1:K)
{
  print(paste("Creating gene set analysis (GSA) plots for pathway ",k,"/", K, sep=""))
  cat(paste("LOG:Creating gene set analysis (GSA) plots for pathway ",k,"/", K, sep=""),file=log,sep='\n\n',append=TRUE)
  cat(paste("document.write('<p>Creating gene set analysis (GSA) plots for pathway ",k,"/", K, "</p>');", sep=""), file=paste(path.inc,"//status.js",sep=""),append=TRUE)

#add menu per pathway for each covariate
  strTemp=paste("document.write('                <div class=\"content\" id=\"",gene.set.names[k],"-GSA\">');\n",sep="")
  cat(strTemp,file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)

  cat("document.write('	 			   <nav class=\"top-bar\" data-topbar>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
  cat("document.write('	 			         <ul class=\"button-group  tabs\" data-tab>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)


##GSAMENU
  # first loop to create menu
   catlevels<-covariates
  for(i in 1:length(predictors))
  {
		kk=i
		preTemp<-predictors[i]
		
		numlevels=0
		tempLevels<-c()
		tempLevelNames<-c()
		#need to check how many levels a covariate has, and also check for 
		for(j in 1:length(catlevels))	
		{
			test1 = gsub(preTemp,"",catlevels[j])
			if(is.element(paste(preTemp,test1,sep=""),catlevels))
			{
				#is a valid catlevel 
				numlevels = numlevels+1
				
				testNumeric=paste(catlevels[j],sep="")
				y<-charToRaw(testNumeric)
				stringNumeric=paste(y, collapse = '')

				tempLevels<-c(tempLevels,stringNumeric)
				tempLevelNames<-c(tempLevelNames,test1)
				
			}
		}
		

		lvlTemp <-tempLevels[1]


		#decrement the 'active' counter if covariate is not valid
		if(numlevels==0)
		{
		  kk=kk-1
		}

		if(numlevels==1)#continuous or categorical w/1 entry or boolean
		{
		if(kk==1)
		{
			strTemp=paste("document.write('		                 <li class=\"tab-title active\"><a href=\"#",gene.set.names[k],"-GSA-",lvlTemp,"\">",predictors[i],"</a></li>');\n",sep="")
		}else{
			strTemp=paste("document.write('		                 <li class=\"tab-title\"><a href=\"#",gene.set.names[k],"-GSA-",lvlTemp,"\">",predictors[i],"</a></li>');\n",sep="")
		}
		cat(strTemp,file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
		}
		if(numlevels>1)#categorical
		{
			#add top level menu the submenu

			preTemp = gsub("\\.","",preTemp)
			preTemp = gsub("\\s","",preTemp)
		if(kk==1)
		{
			strTemp=paste("document.write('		                 <li class=\"tab-title active\"><a href=\"#",gene.set.names[k],"-GSA-",preTemp,"\" data-dropdown=\"dd",gene.set.names[k],preTemp,"\">",predictors[i],"</a></li>');\n",sep="")
		}else{
			strTemp=paste("document.write('		                 <li class=\"tab-title\"><a href=\"#",gene.set.names[k],"-GSA-",preTemp,"\" data-dropdown=\"dd",gene.set.names[k],preTemp,"\">",predictors[i],"</a></li>');\n",sep="")
			}
			cat(strTemp,file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)	

			strTemp=paste("document.write('	  	                   <ul id=\"dd",gene.set.names[k],preTemp,"\" class=\"f-dropdown\" data-dropdown-content data-tab>');\n",sep="")
			cat(strTemp,file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)	

			for(lvl in 1:numlevels)
			{

				lvlTemp <-tempLevels[lvl]

				strTemp=paste("document.write('	    	                     <li><a href=\"#",gene.set.names[k],"-GSA-",lvlTemp,"\">",tempLevelNames[lvl],"</a></li>');\n",sep="")
				cat(strTemp,file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)	

			}
			cat("document.write('	  	                   </ul>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
		}


    }
    cat("document.write('	 			         </ul>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
    cat("document.write('	 			   </nav>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
    
    ## end NAV menu for pathway - start div for pathway content
    cat("document.write('		          <div class=\"tabs-content\">');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
	

 # for each covariate saved in DEresults
  for(i in 1:nvars)
  {  

	catTemp <-covariates[i]
	y<-charToRaw(catTemp)
	strNumeric=paste(y, collapse = '')

#start of individual content - modify to make target for menu
    if(i==1)
	{
    		strTemp=paste("document.write('                    <div class=\"content active\" id=\"",gene.set.names[k],"-GSA-",strNumeric,"\">');\n",sep="")
	}else{
		strTemp=paste("document.write('                    <div class=\"content\" id=\"",gene.set.names[k],"-GSA-",strNumeric,"\">');\n",sep="")
	}
     cat(strTemp,file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)

     cat("document.write('        		     <ul class=\"small-block-grid-1\">');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)

    pvals.all = pvals[,i]
    tstats.all = ests[,i]/DEresults$ses[,i]
    

      pvals.path = pvals.all[gene.sets[,k]==1]
      
      # calculate global significance stat:
      global.sig.stats[k,i] = round(sqrt(mean((tstats.all^2)[gene.sets[,k]==1])),4)
      global.sig.stats.updown[k,i] = mean((sign(tstats.all)*(tstats.all^2))[gene.sets[,k]==1])
      global.sig.stats.updown[k,i] = round(sign(global.sig.stats.updown[k,i])*sqrt(abs(global.sig.stats.updown[k,i])),4)
      
      if(saveresults)
      {
        # draw a volcano plot
        for(r in 1:length(plottypearg)){
          plottype=plottypearg[r];
          tempfilename = drawplot(paste(path.to.GSA.results,"//volcano plot - ",covariates[i]," - ",gene.set.names[k],sep=""),plottype,width=1.3,height=1.3)
		tempfilename=gsub(path.results,"results",tempfilename)
		if(r==1)
		{
		 	#BEGIN DYNAMIC HTML - plot by covariate
			strTemp=paste("document.write('                        <li>Volcano Plot - ",covariates[i],"<br><img src=\"",tempfilename,".png\"></img></li>');\n",sep="")
			cat(strTemp,file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
			#END DYNAMIC HTML
		}

        volcanocol = rep("grey",ngenes)
        volcanocol[gene.sets[,k]==1]=codecols2[3]
        volcanopch = rep(1,ngenes)
        volcanopch[gene.sets[,k]==1]=16
		plot(DEresults$ests[,i],-log10(pvals[,i]),xlab="Log(fold change)",ylab = "-log10(p-value)",main="",col=volcanocol,pch=volcanopch)   
        abline(h=-log10(c(0.01,0.001)),lty=2)
        points(DEresults$ests[gene.sets[,k]==1,i],-log10(pvals[gene.sets[,k]==1,i]),pch=16,col=codecols2[3])
        legend("bottomright",pch=c(16,16),col=c(codecols2[3]),legend = paste(gene.set.names[k],"pathway genes"))
        sig.and.in.path = (pvals.all<0.01)&(gene.sets[,k]==1)
        if(sum(sig.and.in.path)>0){text(x=ests[sig.and.in.path,i],y=-log10(pvals[sig.and.in.path,i]),labels=dimnames(ests)[[1]][sig.and.in.path],cex=.8)}
        dev.off()}
        
        # write a csv of the top genes in the pathway:
        select1 = (gene.sets[,k]==1)
        select2 = order(pvals.path,decreasing=FALSE)[1:10]
        select3 = (((1:length(pvals.all))[select1])[select2])
        out = cbind(DEresults$ests[,i],DEresults$ests[,i]-1.96*DEresults$ses[,i],DEresults$ests[,i]+1.96*DEresults$ses[,i],
                    DEresults$pvals[,i],as.data.frame(DEresults$Pathways))
        extracolumn=FALSE
        if(pval.adjustment=="BY")
        {
          extracolumn=TRUE
          pvals.a = p.adjust(pvals[,i],method="BY")
          out = cbind(out,pvals.a); dimnames(out)[[2]][6] = "FDR"
        }
        if(pval.adjustment=="BH")
        {
          extracolumn=TRUE
          pvals.a = p.adjust(pvals[,i],method="BH")
          out = cbind(out,pvals.a); dimnames(out)[[2]][6] = "FDR"
        }
        if(pval.adjustment=="Bonferroni")
        {
          extracolumn=TRUE
          pvals.a = p.adjust(pvals[,i],method="bonferroni")
          out = cbind(out,pvals.a); dimnames(out)[[2]][6] = "Bonf. p-value"
        }
        out = out[select3,]
        if(extracolumn){out = out[,c(1:4,6,5)]}
        if(!extracolumn){dimnames(out)[[2]] = c("Log fold change","Lower confidence limit","Upper confidence limit","P-value","Pathways")}
        if(extracolumn){dimnames(out)[[2]] = c("Log fold change","Lower confidence limit","Upper confidence limit","P-value",dimnames(out)[[2]][5],"Pathways")}
		    out[,c(1:3)]=signif(out[,c(1:3)],3)
	    	out[,4]=signif(out[,4],4)
		    if(extracolumn){out[,5]=signif(out[,5],4)} 
    
        write.csv(out,file=correct.filename(paste(path.to.GSA.results,"//Top 10 genes in ",dimnames(gene.sets)[[2]][k]," - ",dimnames(DEresults$ses)[[2]][i],".csv",sep="")))
	   paste
	   numRows=nrow(out)
	   numCols=ncol(out)
	   genes=rownames(out)	
	   cols=dimnames(out)[[2]]
	   
    
 
	  #HTML for table
	   strTemp=paste("document.write('                        <li>Top 10 genes in ",dimnames(gene.sets)[[2]][k]," - ",dimnames(DEresults$ses)[[2]][i],"<br>');\n",sep="")
	   cat(strTemp,file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
#TABLE GOES HERE


	  if(numCols==6)
	  {
	  	outtable=xtable(out,display=c("s","g","g","g","g","g","s"))
	  }else{
		outtable=xtable(out,display=c("s","g","g","g","g","s"))
	  }
        tabletext=print(outtable,type="html",file="",print.results=FALSE,timestamp=NULL,comment=FALSE,html.table.attributes="")
	   	 
	  tabletext=gsub("<table >","document.write('<table>');\n",tabletext)
	  tabletext=gsub("<tr>","document.write('<tr>",tabletext)
	  tabletext=gsub("</tr>","</tr>');\n",tabletext)
	  tabletext=gsub("</table>","document.write('</table>');\n",tabletext)
	  cat(tabletext,file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)

	   cat("document.write('                        </li>')\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
#END HTML placeholder

      } #end if(saveresults)
	  cat("document.write('                     </ul>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
	  cat("document.write('                    </div>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
    }

  cat("document.write('               </div>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
  cat("document.write('               </div>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)	
  }
  

  if(saveresults)
  {
  
    print("Creating gene set analysis (GSA) results files")
    cat("LOG:Creating gene set analysis (GSA) results files",file=log,sep='\n\n',append=TRUE)
    cat("document.write('<p>Creating gene set analysis (GSA) results files</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
    
    # write global significance statistics:
    write.csv(round(global.sig.stats,3),file=paste(path.to.GSA.results,"//global significance stats - mean abs t-stat.csv",sep=""))
    write.csv(round(global.sig.stats.updown,3),file=paste(path.to.GSA.results,"//global significance stats - mean t-stat.csv",sep=""))
 
##start summary DIV
    cat("document.write('                   <div class=\"content active\" id=\"summary-GSA\">');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)

    if(dim(global.sig.stats)[2]>1)
    {
	cat("document.write('	 			   <nav class=\"top-bar\" data-topbar>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
	cat("document.write('	 			         <ul class=\"button-group  tabs\" data-tab>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)

	cat("document.write('						   <li class=\"tab-title active\"><a href=\"#summary-GSA-Global\">Global Sig Scores</a></li>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
	cat("document.write('						   <li class=\"tab-title \"><a href=\"#summary-GSA-Directed\">Directed Global Sig Scores</a></li>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)

	cat("document.write('	 			         </ul>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
	cat("document.write('	 			   </nav>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
	cat("document.write('		             <div class=\"tabs-content\">');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
	


      ## heatmaps of path significance scores
      for(r in 1:length(plottypearg)){
	    
      plottype=plottypearg[r];
      tempfilename = drawplot(paste(path.to.GSA.results,"//heatmap of global significance scores",sep=""),plottype,width=1.3,height=1.3)
	 tempfilename=gsub(path.results,"results",tempfilename)
	 if(r==1)
	{
	 cat("document.write('         	          <div class=\"content active\" id=\"summary-GSA-Global\">');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
	 cat("document.write('        		       <ul class=\"small-block-grid-1\">');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)

	 strTemp=paste("document.write('              	    				<li>Global Significance Scores<br><img src=\"",tempfilename,".png\"></img></li>');\n",sep="")
	 cat(strTemp,file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
	 cat("document.write('     			      </ul>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)	
	 cat("document.write('                     </div>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)


	}



      par(mar=c(12,4,2,1))
      heatmap.2(global.sig.stats,scale="none",symm=FALSE,trace="none",density.info="none",Colv=TRUE,margin=c(8,8),
                cexCol = 0.2 + 1/log10(dim(global.sig.stats)[1]))
      dev.off()
      tempfilename = drawplot(paste(path.to.GSA.results,"//heatmap of directed global significance scores - directed",sep=""),plottype,width=1.3,height=1.3)
	tempfilename=gsub(path.results,"results",tempfilename)
	if(r==1)
	{
	cat("document.write('         	          <div class=\"content \" id=\"summary-GSA-Directed\">');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
	cat("document.write('        		      <ul class=\"small-block-grid-1\">');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)

	strTemp=paste("document.write('              	    				<li>Directed Global Significance Scores<br><img src=\"",tempfilename,".png\"></img></li>');\n",sep="")
	cat(strTemp,file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
	cat("document.write('     			     </ul>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)	
	cat("document.write('                   </div>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)

	}



      par(mar=c(16,4,2,1))
      hmcols<-colorRampPalette(c("blue","antiquewhite3",codecols2[4]))(256)
      breaks = seq(-max(abs(global.sig.stats.updown)),max(abs(global.sig.stats.updown)),length.out=length(hmcols)+1)
      heatmap.2(global.sig.stats.updown,scale="none",symm=FALSE,trace="none",density.info="none",Colv=TRUE,col=hmcols,
                breaks=breaks,margin=c(8,8),cexCol = 0.2 + 1/log10(dim(global.sig.stats)[1]))
      dev.off()
    }
  	cat("document.write('                 </div>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)

  } 
print("Creating gene set analysis (GSA) HTML infrastructure")
cat("LOG:Creating gene set analysis (GSA) HTML infrastructure",file=log,sep='\n\n',append=TRUE)
cat("document.write('<p>Creating gene set analysis (GSA) HTML infrastructure</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)





#START HTML TABLE
numRow=nrow(global.sig.stats)
numCol=ncol(global.sig.stats)

genes=rownames(global.sig.stats)	
cols=dimnames(global.sig.stats)[[2]]

if(!dim(global.sig.stats)[2]>1)
{
	
	cat("document.write('         	          <div class=\"content active\" id=\"summary-GSA-MeanAbs\">');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
	cat("document.write('        		       <ul class=\"small-block-grid-2\">');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)





#output HTML tables here - 
	 strTemp=paste("document.write('              	    				<li>Global Significance Stats - Mean Abs T-Stat<br>');\n",sep="")
	 cat(strTemp,file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)

        tabletext=print(xtable(global.sig.stats),type="html",file="",print.results=FALSE,timestamp=NULL,comment=FALSE,html.table.attributes="")
	   	   
	  tabletext=gsub("<table >","document.write('<table >');\n",tabletext)
	  tabletext=gsub("<tr>","document.write('<tr>",tabletext)
	  tabletext=gsub("</tr>","</tr>');\n",tabletext)
	  tabletext=gsub("</table>","document.write('</table>');\n",tabletext)
	  cat(tabletext,file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)


	   cat("document.write('                        </li>')\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
#END HTML table

#START HTML TABLE
numRow=nrow(global.sig.stats.updown)
numCol=ncol(global.sig.stats.updown)

genes=rownames(global.sig.stats.updown)	
cols=dimnames(global.sig.stats.updown)[[2]]


#output HTML tables here - 
	 strTemp=paste("document.write('              	    				<li>Directed Global Significance Stats - Mean T-Stat<br>');\n",sep="")
	 cat(strTemp,file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)

        tabletext=print(xtable(global.sig.stats.updown),type="html",file="",print.results=FALSE,timestamp=NULL,comment=FALSE,html.table.attributes="")
	   	   
	  tabletext=gsub("<table >","document.write('<table >');\n",tabletext)
	  tabletext=gsub("<tr>","document.write('<tr>",tabletext)
	  tabletext=gsub("</tr>","</tr>');\n",tabletext)
	  tabletext=gsub("</table>","document.write('</table>');\n",tabletext)
	  cat(tabletext,file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)

         cat("document.write('                        </table>')\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
	   cat("document.write('                        </li>')\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
#END HTML table
	cat("document.write('     			     </ul>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)	
	cat("document.write('                   </div>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)

}

#close UL and DIV for summary page
#  cat("document.write('              	    			</ul>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)	
  cat("document.write('                  </div>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
} #end if(saveresults)
    cat("document.write('		    </div>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)
    cat("document.write('		  </div>');\n",file=paste(path.inc,"//panel2_3.js",sep=""),append=TRUE)


out = list(mean.abs.tstats = global.sig.stats,mean.tstats = global.sig.stats.updown)
print("Finished gene set analysis (GSA)")
cat("LOG: Finished gene set analysis (GSA)",file=log,sep='\n\n',append=TRUE)
cat("document.write('<p>Finished gene set analysis (GSA)</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
return(out)
}

#################################
#################################
## pathview
#################################
#################################

# color KEGG maps by differential expression: 
# options:  by fold-change, by signed log p-value, by test stat
run.pathview = function(DEresult, keggIDs=c("04330","04310","04340","04350","04010","04630","04151","04014","04210","04110"),
                        colorscheme = c("tstat","estimate"),pvalthresh=0.05, view=c("KEGG","graphviz"),
                        kegg.dir=NULL,xml.files=NULL,predictors=NULL)   
{



  J = DEresult$npredictors
  keggNames=c("notch","wnt","hh","stat","pi3k","ras","apop","cc","tgfb","mapk")
print("Starting pathview analysis")
cat("LOG:Starting pathview analysis",file=log,sep='\n\n',append=TRUE)
cat("document.write('<p>Starting pathview analysis</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)

### PATHVIEW HTML CODE
cat("document.write('	    <div class=\"large-2 columns\" id=\"side_menu\">');\n",file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)
cat("document.write('    		  <H4>Pathways</H4>');\n",file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)
cat("document.write('              <ul class=\"button-group tabs vertical\" data-tab>');\n",file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title active\"><a href=\"#apop-pathview\">Apoptosis</a></li>');\n",file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#cc-pathview\">Cell Cycle</a></li>');\n",file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#hh-pathview\">Hedgehog</a></li>');\n",file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#stat-pathview\">JAK-STAT</a></li>');\n",file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#mapk-pathview\">MAPK</a></li>');\n",file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#notch-pathview\">Notch</a></li>');\n",file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#pi3k-pathview\">PI3K</a></li>');\n",file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#ras-pathview\">RAS</a></li>');\n",file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#tgfb-pathview\">TGF-Beta</a></li>');\n",file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)
cat("document.write('                <li class=\"tab-title\"><a href=\"#wnt-pathview\">Wnt</a></li>');\n",file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)

if(length(keggIDs) > length(keggNames))
{
	lstart=length(keggNames)+1
	for(x in lstart:length(keggIDs))
	{
		strTemp=paste("document.write('                <li class=\"tab-title\"><a href=\"#hsa",keggIDs[x],"-pathview\">hsa",keggIDs[x],"</a></li>');\n",sep="")
		cat(strTemp,file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)
	}
}

print("Creating pathview HTML infrastructure")
cat("LOG:Creating pathview HTML infrastructure",file=log,sep='\n\n',append=TRUE)
cat("document.write('<p>Creating pathview HTML infrastructure</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)

cat("document.write('              </ul>');\n",file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)
cat("document.write('          </div>');\n",file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)
cat("document.write('		  <div class=\"large-10 columns\">');\n",file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)
cat("document.write('		  	<div class=\"tabs-content\">');\n",file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)




  if(colorscheme=="tstat")
  {
    wts = as.matrix(DEresult$ests[,1:J]/DEresult$ses[,1:J])
  }
  if(colorscheme=="DE")  
  {
    wts = as.matrix(DEresult$ests[,1:J]) 
  }
  # truncate the weights:
  wts = wts*(DEresult$pvals[,1:J]<pvalthresh)
  dimnames(wts)[[2]] = dimnames(DEresult$ests)[[2]][1:J]


  #loop to map pathway/predictor filenames to pathway links
  for(k in 1:length(keggIDs))
  {
	cat(paste(keggIDs[k],keggNames[k],sep=":"),file=log,sep='\n',append=TRUE)
	if(k>length(keggNames))
	{
		keggRef=paste("hsa",keggIDs[k],"-pathview",sep="")
		strTemp=paste("document.write('                   <div class=\"content\" id=\"",keggRef,"\">')\n",sep="")
	}else{
		keggRef=paste(keggNames[k],"-pathview",sep="")

		if(k==7)
		{
			strTemp=paste("document.write('                   <div class=\"content active\" id=\"",keggRef,"\">')\n",sep="")
		}else{
			strTemp=paste("document.write('                   <div class=\"content\" id=\"",keggRef,"\">')\n",sep="")
		}
	}	
	cat(strTemp,file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)
   
	cat("document.write('	 	            <nav class=\"top-bar\" data-topbar>');\n",file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)
	cat("document.write('	 	           	<ul class=\"button-group  tabs\" data-tab>');\n",file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)
   
#Menu Code - covariate on menu, level as dropdown
##PATHVIEWMENU
	catlevels<-dimnames(wts)[[2]]

  	for(i in 1:length(predictors))
  	{
		kk=i
		preTemp<-predictors[i]
		
		numlevels=0
		tempLevels<-c()
		tempLevelNames<-c()
		#need to check how many levels a covariate has, and also check for 
		for(j in 1:length(catlevels))	
		{
			test1 = gsub(preTemp,"",catlevels[j])
			if(is.element(paste(preTemp,test1,sep=""),catlevels))
			{
				#is a valid catlevel 
				numlevels = numlevels+1
				
				testNumeric=paste(catlevels[j],sep="")
				y<-charToRaw(testNumeric)
				stringNumeric=paste(y, collapse = '')

				tempLevels<-c(tempLevels,stringNumeric)
				tempLevelNames<-c(tempLevelNames,test1)
				
			}
		}
		

		lvlTemp <-tempLevels[1]
		
		#decrement the 'active' counter if covariate is not valid
		if(numlevels==0)
		{
		  kk=kk-1
		}

		if(numlevels==1)#continuous
		{
		if(kk==1)
		{
			strTemp=paste("document.write('		                 <li class=\"tab-title active\"><a href=\"#",keggRef,"-",lvlTemp,"\">",predictors[i],"</a></li>');\n",sep="")
		}else{
			strTemp=paste("document.write('		                 <li class=\"tab-title\"><a href=\"#",keggRef,"-",lvlTemp,"\">",predictors[i],"</a></li>');\n",sep="")
		}
		cat(strTemp,file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)
		}
		if(numlevels>1)#categorical
		{
			#add top level menu the submenu

			preTemp = gsub("\\.","",preTemp)
			preTemp = gsub("\\s","",preTemp)
		if(kk==1)
		{
			strTemp=paste("document.write('		                 <li class=\"tab-title active\"><a href=\"#",keggRef,"-",preTemp,"\" data-dropdown=\"dd",keggRef,preTemp,"\">",predictors[i],"</a></li>');\n",sep="")
		}else{
			strTemp=paste("document.write('		                 <li class=\"tab-title\"><a href=\"#",keggRef,"-",preTemp,"\" data-dropdown=\"dd",keggRef,preTemp,"\">",predictors[i],"</a></li>');\n",sep="")
			}
			cat(strTemp,file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)	

			strTemp=paste("document.write('	  	                   <ul id=\"dd",keggRef,preTemp,"\" class=\"f-dropdown\" data-dropdown-content data-tab>');\n",sep="")
			cat(strTemp,file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)	

			for(lvl in 1:numlevels)
			{

				lvlTemp <-tempLevels[lvl]

				strTemp=paste("document.write('	    	                     <li><a href=\"#",keggRef,"-",lvlTemp,"\">",tempLevelNames[lvl],"</a></li>');\n",sep="")
				cat(strTemp,file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)	

			}
			cat("document.write('	  	                   </ul>');\n",file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)
		}
	}

	cat("document.write('	 		          </ul>');\n",file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)
	cat("document.write('	 	            </nav>');\n",file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)
#END OF MENU	
	cat("document.write('		          <div class=\"tabs-content\">');\n",file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)

  	for(j in 1:length(catlevels))
  	{
		catTemp <-catlevels[j]
		y<-charToRaw(catTemp)
		strNumeric=paste(y, collapse = '')
		if(j==1)
		{
			strTemp=paste("document.write('                         <div class=\"content active\" id=\"",keggRef,"-",strNumeric,"\">');\n",sep="")
		}else{
			strTemp=paste("document.write('                         <div class=\"content\" id=\"",keggRef,"-",strNumeric,"\">');\n",sep="")
		}
		cat(strTemp,file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)
		cat("document.write('        		          <ul class=\"small-block-grid-1\">');\n",file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)

		strTemp=paste("document.write('                             <li>Pathview - ",catlevels[j],"<br><img src=\"results/Pathview/hsa",keggIDs[k],".",catlevels[j],".png\"></img></li>');\n",sep="")
		cat(strTemp,file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)
		cat("document.write('        		          </ul>');\n",file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)
		cat("document.write('                   </div>');\n",file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)

	}	
		cat("document.write('              </div>');\n",file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)
    		cat("document.write('          </div>');\n",file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)

  }


    cat("document.write('	   </div>');\n",file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)
    cat("document.write('  </div>');\n",file=paste(path.inc,"//panel2_4.js",sep=""),append=TRUE)

  write.csv(wts,file=paste(path.to.csvs,"//pathview - weights.csv",sep=""))
  write.csv(DEresult$pvals,file=paste(path.to.csvs,"//pathview - pvals.csv",sep=""))
  write.csv(DEresult$ests,file=paste(path.to.csvs,"//pathview - ests.csv",sep=""))

d=FALSE

  maindir = getwd()
  setwd(path.to.pathview.results)
  for(j in 1:J)
  {
    # loop through specified keggIDs:
    if(length(keggIDs)>0)
    {
      for(k in 1:length(keggIDs))
      {
        if(!d){
          print(paste("Downloading KEGG files for", keggIDs[k]," pathway ",k,"/",length(keggIDs)))
          cat(paste("LOG:Downloading KEGG files for", keggIDs[k]," pathway ",k,"/",length(keggIDs)),file=log,sep='\n\n',append=TRUE)
          cat(paste("document.write('<p>Downloading KEGG files for", keggIDs[k]," pathway ",k,"/",length(keggIDs)," </p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
        }
        if(colorscheme=="DE"){genelimit = c(-2.5,2.5)}
        if(colorscheme=="tstat"){genelimit = c(-10,10)}
        
        # run pathview for the appropriate ID:
        if(view=="KEGG")
        {
          pv <- suppressMessages(suppressWarnings(try(pathview(gene.data = wts[,j], cpd.data=NULL,pathway.id = keggIDs[k], out.suffix = dimnames(wts)[[2]][j],
                       kegg.native = TRUE,limit=list(gene=genelimit,cpd=c(-1,1)),gene.idtype="Symbol",multi.state=FALSE,same.layer=TRUE,
                       discrete =list(gene=FALSE, cpd=FALSE)))))
        }
        if(view=="graphviz")
        {
          pv <- suppressMessages(suppressWarnings(try(pathview(gene.data = wts[,j], cpd.data=NULL,pathway.id = keggIDs[k], out.suffix = dimnames(wts)[[2]][j],
                         kegg.native = FALSE,limit=list(gene=genelimit,cpd=c(-1,1)),gene.idtype="Symbol",multi.state=FALSE,same.layer=FALSE))))  
        }
      }
      d=TRUE
      print(paste("Creating pathview analysis results ",j,"/",J))
      cat(paste("LOG: Creating pathview analysis results ",j,"/",J),file=log,sep='\n\n',append=TRUE)
      cat(paste("document.write('<p>Creating pathview analysis results ",j,"/",J," </p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
    }
  }
  setwd(maindir)
print("Finished pathview analysis")
cat("LOG:Finished pathview analysis",file=log,sep='\n\n',append=TRUE)
cat("document.write('<p>Finished pathview analysis</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
}




### plot pathway score and DE results:
summarize.pathscores.and.DE = function(deregresults,gsaresults,DEresults.pathscore,predictors)
{
  print("Finishing analysis")
  cat("LOG:Finishing analysis",file=log,sep='\n\n',append=TRUE)
  cat("document.write('<p>Finishing analysis</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
##SIGPLOT

  if(identical(dimnames(deregresults$scores)[[2]],dimnames(gsaresults$mean.abs.tstats)[[1]]))
  {
    ## for each covariate included in the dereg module and in the DE module, plot the significance scores of each pathway
    pathcovariates=dimnames(gsaresults$mean.abs.tstats)[[2]]

  # create SigPlots include file 
	cat("document.write('		<div class=\"large-12 columns\">');\n",file=paste(path.inc,"//panel2_8.js",sep=""),append=TRUE)
	cat("document.write('	 	  <nav class=\"top-bar\" data-topbar>');\n",file=paste(path.inc,"//panel2_8.js",sep=""),append=TRUE)
	cat("document.write('	 		<ul class=\"button-group  tabs\" data-tab>');\n",file=paste(path.inc,"//panel2_8.js",sep=""),append=TRUE)
   
#loop to make menu
##SIGMENU
catlevels<-pathcovariates

  for(i in 1:length(predictors))
  {
		kk=i
		preTemp<-predictors[i]
		
		numlevels=0
		tempLevels<-c()
		tempLevelNames<-c()
		#need to check how many levels a covariate has, and also check for 
		for(j in 1:length(catlevels))	
		{
			test1 = gsub(preTemp,"",catlevels[j])
			if(is.element(paste(preTemp,test1,sep=""),catlevels))
			{
				#is a valid catlevel 
				numlevels = numlevels+1
				
				testNumeric=paste(catlevels[j],sep="")
				y<-charToRaw(testNumeric)
				stringNumeric=paste(y, collapse = '')

				tempLevels<-c(tempLevels,stringNumeric)
				tempLevelNames<-c(tempLevelNames,test1)
				
			}
		}
		

		lvlTemp <-tempLevels[1]


		#decrement the 'active' counter if covariate is not valid
		if(numlevels==0)
		{
		  kk=kk-1
		}

		if(numlevels==1)#continuous
		{
		if(kk==1)
		{
			strTemp=paste("document.write('		                 <li class=\"tab-title active\"><a href=\"#SP-",lvlTemp,"\">",predictors[i],"</a></li>');\n",sep="")
		}else{
			strTemp=paste("document.write('		                 <li class=\"tab-title\"><a href=\"#SP-",lvlTemp,"\">",predictors[i],"</a></li>');\n",sep="")
		}
		cat(strTemp,file=paste(path.inc,"//panel2_8.js",sep=""),append=TRUE)
		}
		if(numlevels>1)#categorical
		{
			#add top level menu the submenu

			preTemp = gsub("\\.","",preTemp)
			preTemp = gsub("\\s","",preTemp)
		if(kk==1)
		{
			strTemp=paste("document.write('		                 <li class=\"tab-title active\"><a href=\"#SP-",preTemp,"\" data-dropdown=\"ddSP",preTemp,"\">",predictors[i],"</a></li>');\n",sep="")
		}else{
			strTemp=paste("document.write('		                 <li class=\"tab-title\"><a href=\"#SP-",preTemp,"\" data-dropdown=\"ddSP",preTemp,"\">",predictors[i],"</a></li>');\n",sep="")
			}
			cat(strTemp,file=paste(path.inc,"//panel2_8.js",sep=""),append=TRUE)	

			strTemp=paste("document.write('	  	                   <ul id=\"ddSP",preTemp,"\" class=\"f-dropdown\" data-dropdown-content data-tab>');\n",sep="")
			cat(strTemp,file=paste(path.inc,"//panel2_8.js",sep=""),append=TRUE)	

			for(lvl in 1:numlevels)
			{

				lvlTemp <-tempLevels[lvl]
				lvlTemp = gsub("\\.","",lvlTemp)
				lvlTemp = gsub("\\s","",lvlTemp)
				strTemp=paste("document.write('	    	                     <li><a href=\"#SP-",lvlTemp,"\">",tempLevelNames[lvl],"</a></li>');\n",sep="")
				cat(strTemp,file=paste(path.inc,"//panel2_8.js",sep=""),append=TRUE)	

			}
			cat("document.write('	  	                   </ul>');\n",file=paste(path.inc,"//panel2_8.js",sep=""),append=TRUE)
		}
}
	cat("document.write('	 		   </ul>');\n",file=paste(path.inc,"//panel2_8.js",sep=""),append=TRUE)
	cat("document.write('	 	  </nav>');\n",file=paste(path.inc,"//panel2_8.js",sep=""),append=TRUE)
	#END OF MENU

	cat("document.write('          <div class=\"tabs-content\">');\n",file=paste(path.inc,"//panel2_8.js",sep=""),append=TRUE)

    for(i in 1:length(pathcovariates))
    {
     cov1 = pathcovariates[i]
	catTemp <-cov1
	y<-charToRaw(catTemp)
	strNumeric=paste(y, collapse = '')

	if(i==1)
	{
 		strTemp=paste("document.write('             <div class=\"content active\" id=\"SP-",strNumeric,"\">');\n",sep="")
	}else{
		strTemp=paste("document.write('             <div class=\"content\" id=\"SP-",strNumeric,"\">');",sep="")
	}

	cat(strTemp,file=paste(path.inc,"//panel2_8.js",sep=""),append=TRUE)
	cat("document.write('		  	<ul class=\"small-block-grid-1\">');\n",file=paste(path.inc,"//panel2_8.js",sep=""),append=TRUE)


      for(r in 1:length(plottypearg)){
        plottype=plottypearg[r];
        tempfilename = drawplot(paste(path.results,"//","pathways significance plot - ",cov1,sep=""),plottype)
	   tempfilename=gsub(path.results,"results",tempfilename)
	   if(r==1)
	   {
          strTemp=paste("document.write('		  	  <li>Pathway Significance Plot - ",cov1,"<br><img src=\"",tempfilename,".png\"></img></li>');\n",sep="")
	     cat(strTemp,file=paste(path.inc,"//panel2_8.js",sep=""),append=TRUE)
	   }


        par(mar = c(5,6,2,1))
        plot(-log10(DEresults.pathscore$pvals[,cov1]),gsaresults$mean.abs.tstats[,cov1],xlab=paste("-log10(p-value) of pathway score vs.\n",cov1),
             ylab=paste("Global significance statistic for pathways vs.\n",cov1),pch=16,col=codecols2[1],cex=4)
        text(-log10(DEresults.pathscore$pvals[,cov1]),gsaresults$mean.abs.tstats[,cov1],dimnames(deregresults$scores)[[2]])
        dev.off() 

	 } 

  	  cat("document.write('		  	</ul>');\n",file=paste(path.inc,"//panel2_8.js",sep=""),append=TRUE)
  	  cat("document.write('		  </div>');\n",file=paste(path.inc,"//panel2_8.js",sep=""),append=TRUE) 

    }


  	cat("document.write('	   </div>');\n",file=paste(path.inc,"//panel2_8.js",sep=""),append=TRUE)  
  	cat("document.write('	  </div>');\n",file=paste(path.inc,"//panel2_8.js",sep=""),append=TRUE)  
  }

  if(!identical(dimnames(deregresults$scores)[[2]],dimnames(gsaresults$mean.abs.tstats)[[1]]))
  {
    # for each covariate, plot both: (or just plot the predefined scores) 
    for(r in 1:length(plottypearg)){
      plottype=plottypearg[r];
      tempfilename = drawplot(paste(path.results,"//","pathways significance plot - ",cov1,sep=""),plottype,width=1.5)
	 tempfilename=gsub(path.results,"results",tempfilename)
      par(mfrow=c(1,2))
      # plot GSA results:
      stats = gsaresults$mean.abs.tstats[,cov1]
      plot(stats[order(stats)],xlab="",ylab=paste("Global significance statistic for pathways vs.",cov1),
           pch=16,col=codecols2[1],cex=4,xaxt="n")
      text(1:length(gsaresults$mean.abs.tstats),stats[order(stats)],dimnames(gsaresults$mean.abs.tstats)[[1]][order(stats)])
      
      # plot dereg results:
      stats = -log10(DEresults.pathscore$pvals[,cov1])
      plot(stats[order(stats)],xlab="",
           ylab=paste("-log10(p-value) of pathway score vs.",cov1),pch=16,col=codecols2[2],cex=4,xaxt="n")
      text(1:length(stats),stats[order(stats)], dimnames(DEresults.pathscore$pvals)[[1]][order(DEresults.pathscore$pvals[,cov1],decreasing=TRUE)])
      
      dev.off()}
  }
}


# function to test internet connection for pathview:
testCon <- function(url = "http://www.google.com") {
  
  # test the http capabilities of the current R build
  http <- as.logical(capabilities(what = "http/ftp"))
  if (!http) return(FALSE)
  
  # test connection by trying to read first line at url
  test <- try(suppressWarnings(readLines(url, n = 1)), silent = TRUE)  # silent errors
  
  # return FALSE if test is class 'try-error'
  ifelse(inherits(test, "try-error"), FALSE, TRUE)
}



heatmap.plus = function (x, Rowv = NULL, Colv = if (symm) "Rowv" else NULL, 
          distfun = dist, hclustfun = hclust, reorderfun = function(d, 
                                                                    w) reorder(d, w), add.expr, symm = FALSE, revC = identical(Colv, 
                                                                                                                               "Rowv"), scale = c("row", "column", "none"), na.rm = TRUE, 
          margins = c(5, 5), ColSideColors, RowSideColors, cexRow = 0.2 + 
            1/log10(nr), cexCol = 0.2 + 1/log10(nc), labRow = NULL, 
          labCol = NULL, main = NULL, xlab = NULL, ylab = NULL, keep.dendro = FALSE, 
          verbose = getOption("verbose"), ...) 
{
  scale <- if (symm && missing(scale)) 
    "none"
  else match.arg(scale)
  if (length(di <- dim(x)) != 2 || !is.numeric(x)) 
    stop("'x' must be a numeric matrix")
  nr <- di[1]
  nc <- di[2]
  if (nr <= 1 || nc <= 1) 
    stop("'x' must have at least 2 rows and 2 columns")
  if (!is.numeric(margins) || length(margins) != 2) 
    stop("'margins' must be a numeric vector of length 2")
  doRdend <- !identical(Rowv, NA)
  doCdend <- !identical(Colv, NA)
  if (is.null(Rowv)) 
    Rowv <- rowMeans(x, na.rm = na.rm)
  if (is.null(Colv)) 
    Colv <- colMeans(x, na.rm = na.rm)
  if (doRdend) {
    if (inherits(Rowv, "dendrogram")) 
      ddr <- Rowv
    else {
      hcr <- hclustfun(distfun(x))
      ddr <- as.dendrogram(hcr)
      if (!is.logical(Rowv) || Rowv) 
        ddr <- reorderfun(ddr, Rowv)
    }
    if (nr != length(rowInd <- order.dendrogram(ddr))) 
      stop("row dendrogram ordering gave index of wrong length")
  }
  else rowInd <- 1:nr
  if (doCdend) {
    if (inherits(Colv, "dendrogram")) 
      ddc <- Colv
    else if (identical(Colv, "Rowv")) {
      if (nr != nc) 
        stop("Colv = \"Rowv\" but nrow(x) != ncol(x)")
      ddc <- ddr
    }
    else {
      hcc <- hclustfun(distfun(if (symm) 
        x
        else t(x)))
      ddc <- as.dendrogram(hcc)
      if (!is.logical(Colv) || Colv) 
        ddc <- reorderfun(ddc, Colv)
    }
    if (nc != length(colInd <- order.dendrogram(ddc))) 
      stop("column dendrogram ordering gave index of wrong length")
  }
  else colInd <- 1:nc
  x <- x[rowInd, colInd]
  labRow <- if (is.null(labRow)) 
    if (is.null(rownames(x))) 
      (1:nr)[rowInd]
  else rownames(x)
  else labRow[rowInd]
  labCol <- if (is.null(labCol)) 
    if (is.null(colnames(x))) 
      (1:nc)[colInd]
  else colnames(x)
  else labCol[colInd]
  if (scale == "row") {
    x <- sweep(x, 1, rowMeans(x, na.rm = na.rm))
    sx <- apply(x, 1, sd, na.rm = na.rm)
    x <- sweep(x, 1, sx, "/")
  }
  else if (scale == "column") {
    x <- sweep(x, 2, colMeans(x, na.rm = na.rm))
    sx <- apply(x, 2, sd, na.rm = na.rm)
    x <- sweep(x, 2, sx, "/")
  }
  lmat <- rbind(c(NA, 3), 2:1)
  lwid <- c(if (doRdend) 1 else 0.05, 4)
  lhei <- c((if (doCdend) 1 else 0.05) + if (!is.null(main)) 0.2 else 0, 4)
  if (!missing(ColSideColors)) {
    if (!is.matrix(ColSideColors)) 
      stop("'ColSideColors' must be a matrix")
    if (!is.character(ColSideColors) || dim(ColSideColors)[1] != 
          nc) 
      stop("'ColSideColors' dim()[2] must be of length ncol(x)")
    lmat <- rbind(lmat[1, ] + 1, c(NA, 1), lmat[2, ] + 1)
    #lhei <- c(lhei[1], 0.2, lhei[2])
    # new code to widen the color bars:
    lhei=c(lhei[1], 0.1*dim(ColSideColors)[2], lhei[2])
    
    
  }
  if (!missing(RowSideColors)) {
    if (!is.matrix(RowSideColors)) 
      stop("'RowSideColors' must be a matrix")
    if (!is.character(RowSideColors) || dim(RowSideColors)[1] != 
          nr) 
      stop("'RowSideColors' must be a character vector of length nrow(x)")
    lmat <- cbind(lmat[, 1] + 1, c(rep(NA, nrow(lmat) - 1), 
                                   1), lmat[, 2] + 1)
    #lwid <- c(lwid[1], 0.2, lwid[2])
    # new code to widen the color bars:
    lwid=c(lwid[1], 0.1*dim(RowSideColors)[2], lwid[2])
    
  }
  lmat[is.na(lmat)] <- 0
  if (verbose) {
    cat("layout: widths = ", lwid, ", heights = ", lhei, 
        "; lmat=\n")
    cat(lmat)
  }
  op <- par(no.readonly = TRUE)
  on.exit(par(op))
  layout(lmat, widths = lwid, heights = lhei, respect = FALSE)
  if (!missing(RowSideColors)) {
    par(mar = c(margins[1], 0, 0, 0.5))
    rsc = RowSideColors[rowInd, ]
    rsc.colors = matrix()
    rsc.names = names(table(rsc))
    rsc.i = 1
    for (rsc.name in rsc.names) {
      rsc.colors[rsc.i] = rsc.name
      rsc[rsc == rsc.name] = rsc.i
      rsc.i = rsc.i + 1
    }
    rsc = matrix(as.numeric(rsc), nrow = dim(rsc)[1])
    image(t(rsc), col = as.vector(rsc.colors), axes = FALSE)
    if (length(colnames(RowSideColors)) > 0) {
      axis(1, 0:(dim(rsc)[2] - 1)/(dim(rsc)[2] - 1), colnames(RowSideColors), 
           las = 2, tick = FALSE)
    }
  }
  if (!missing(ColSideColors)) {
    par(mar = c(0.5, 0, 0, margins[2]))
    csc = ColSideColors[colInd, ]
    csc.colors = matrix()
    csc.names = names(table(csc))
    csc.i = 1
    for (csc.name in csc.names) {
      csc.colors[csc.i] = csc.name
      csc[csc == csc.name] = csc.i
      csc.i = csc.i + 1
    }
    csc = matrix(as.numeric(csc), nrow = dim(csc)[1])
    image(csc, col = as.vector(csc.colors), axes = FALSE)
    if (length(colnames(ColSideColors)) > 0) {
      axis(2, 0:(dim(csc)[2] - 1)/(dim(csc)[2] - 1), colnames(ColSideColors), 
           las = 2, tick = FALSE)
    }
  }
  par(mar = c(margins[1], 0, 0, margins[2]))
  if (!symm || scale != "none") {
    x <- t(x)
  }
  if (revC) {
    iy <- nr:1
    ddr <- rev(ddr)
    x <- x[, iy]
  }
  else iy <- 1:nr
  image(1:nc, 1:nr, x, xlim = 0.5 + c(0, nc), ylim = 0.5 + 
          c(0, nr), axes = FALSE, xlab = "", ylab = "", ...)
  axis(1, 1:nc, labels = labCol, las = 2, line = -0.5, tick = 0, 
       cex.axis = cexCol)
  if (!is.null(xlab)) 
    mtext(xlab, side = 1, line = margins[1] - 1.25)
  axis(4, iy, labels = labRow, las = 2, line = -0.5, tick = 0, 
       cex.axis = cexRow)
  if (!is.null(ylab)) 
    mtext(ylab, side = 4, line = margins[2] - 1.25)
  if (!missing(add.expr)) 
    eval(substitute(add.expr))
  par(mar = c(margins[1], 0, 0, 0))
  if (doRdend) 
    plot(ddr, horiz = TRUE, axes = FALSE, yaxs = "i", leaflab = "none")
  else frame()
  par(mar = c(0, 0, if (!is.null(main)) 1 else 0, margins[2]))
  if (doCdend) 
    plot(ddc, axes = FALSE, xaxs = "i", leaflab = "none")
  else if (!is.null(main)) 
    frame()
  if (!is.null(main)) 
    title(main, cex.main = 1.5 * op[["cex.main"]])
  invisible(list(rowInd = rowInd, colInd = colInd, Rowv = if (keep.dendro && 
                                                                doRdend) ddr, Colv = if (keep.dendro && doCdend) ddc))  
}
